-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_Polygon"

-- **************************************************
-- General information about this script
-- **************************************************

LM_Polygon = {}

function LM_Polygon:Name()
	return "Polygon"
end

function LM_Polygon:Version()
	return "5.0"
end

function LM_Polygon:Description()
	return "Creates a polygon shape in the current layer, or a new layer."
end

function LM_Polygon:Creator()
	return "Lost Marble"
end

function LM_Polygon:UILabel()
	return("Polygon...")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_Polygon.numPoints = 5
LM_Polygon.newLayer = false

-- **************************************************
-- Polygon dialog
-- **************************************************

local LM_PolygonDialog = {}

LM_PolygonDialog.UPDATE = MOHO.MSG_BASE

function LM_PolygonDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Polygon", LM_PolygonDialog)
	local l = d:GetLayout()

	d.moho = moho

	d.preview = MOHO.MeshPreview(200, 200)
	l:AddChild(d.preview)

	l:PushH()
		l:PushV()
			l:AddChild(LM.GUI.StaticText("Number of points"), LM.GUI.ALIGN_LEFT)
		l:Pop()
		l:PushV()
			d.numPoints = LM.GUI.TextControl(0, "0.0000", LM_PolygonDialog.UPDATE, LM.GUI.FIELD_UINT)
			d.numPoints:SetConstantMessages(true)
			l:AddChild(d.numPoints)
		l:Pop()
	l:Pop()

	d.newLayer = LM.GUI.CheckBox("Create polygon in new layer")
	l:AddChild(d.newLayer, LM.GUI.ALIGN_LEFT)

	return d
end

function LM_PolygonDialog:UpdateWidgets()
	self.numPoints:SetValue(LM_Polygon.numPoints)
	self.newLayer:SetValue(LM_Polygon.newLayer)
	if (self.moho.layer:LayerType() == MOHO.LT_VECTOR) then
		self.newLayer:Enable(true)
	else
		self.newLayer:Enable(false)
	end
	self:UpdatePreview()
end

function LM_PolygonDialog:OnValidate()
	local b = true
	if (not self:Validate(self.numPoints, 3, 20)) then
		b = false
	end
	return b
end

function LM_PolygonDialog:HandleMessage(what)
	if (what == self.UPDATE) then
		self:UpdatePreview()
	end
end

function LM_PolygonDialog:OnOK()
	LM_Polygon.numPoints = self.numPoints:IntValue()
	LM_Polygon.newLayer = self.newLayer:Value()
end

function LM_PolygonDialog:UpdatePreview()
	local mesh = self.preview:Mesh()
	mesh:Clear()

	local numPoints = self.numPoints:IntValue()
	numPoints = LM.Clamp(numPoints, 3, 20)

	LM_Polygon:BuildPolygon(mesh, numPoints)
	self.preview:CreateShape(true)
	self.preview:Refresh()
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_Polygon:IsEnabled(moho)
	if (moho.layer:CurrentAction() ~= "") then
		return false -- creating new objects in the middle of an action can lead to unexpected results
	end
	return true
end

function LM_Polygon:Run(moho)
	local dlog = LM_PolygonDialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end

	if (LM_Polygon.newLayer or (moho.layer:LayerType() ~= MOHO.LT_VECTOR)) then
		moho.document:PrepUndo(nil)
		local layer = moho:CreateNewLayer(MOHO.LT_VECTOR)
		layer:SetName("Polygon")
	else
		moho.document:PrepUndo(moho.layer)
	end
	moho.document:SetDirty()

	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	self:BuildPolygon(mesh, self.numPoints)
	moho:CreateShape(true)
	mesh:AddGroup("Polygon")
end

function LM_Polygon:BuildPolygon(mesh, numPoints)
	local oldNumPoints = mesh:CountPoints()
	local v = LM.Vector2:new_local()
	local angle = math.pi / 2
	local dAngle = (math.pi * 2) / numPoints
	local bigR = 0.75

	mesh:SelectNone()

	for i = 1, numPoints do
		v.x = bigR * math.cos(angle)
		v.y = bigR * math.sin(angle)
		if (i == 1) then
			mesh:AddLonePoint(v, 0)
		else
			mesh:AppendPoint(v, 0)
		end
		angle = angle + dAngle
	end

	v.x = bigR * math.cos(angle)
	v.y = bigR * math.sin(angle)
	mesh:AppendPoint(v, 0)

	mesh:WeldPoints(oldNumPoints, oldNumPoints + numPoints, 0)

	for i = 0, numPoints - 1 do
		mesh:Point(oldNumPoints + i):SetCurvature(MOHO.PEAKED, 0)
	end

	mesh:SelectConnected()
end
