-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_Star"

-- **************************************************
-- General information about this script
-- **************************************************

LM_Star = {}

function LM_Star:Name()
	return "Star"
end

function LM_Star:Version()
	return "5.0"
end

function LM_Star:Description()
	return "Creates a star shape in the current layer, or a new layer."
end

function LM_Star:Creator()
	return "Lost Marble"
end

function LM_Star:UILabel()
	return("Star...")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_Star.numPoints = 5
LM_Star.innerRadius = 0.2864745
LM_Star.newLayer = false

-- **************************************************
-- Star dialog
-- **************************************************

local LM_StarDialog = {}

LM_StarDialog.UPDATE = MOHO.MSG_BASE

function LM_StarDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Star", LM_StarDialog)
	local l = d:GetLayout()

	d.moho = moho

	d.preview = MOHO.MeshPreview(200, 200)
	l:AddChild(d.preview)

	l:PushH()
		l:PushV()
			l:AddChild(LM.GUI.StaticText("Number of points"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Inner radius"), LM.GUI.ALIGN_LEFT)
		l:Pop()
		l:PushV()
			d.numPoints = LM.GUI.TextControl(0, "0.0000", LM_StarDialog.UPDATE, LM.GUI.FIELD_UINT)
			d.numPoints:SetConstantMessages(true)
			l:AddChild(d.numPoints)
			d.innerRadius = LM.GUI.TextControl(0, "0.0000", LM_StarDialog.UPDATE, LM.GUI.FIELD_UFLOAT)
			d.innerRadius:SetWheelInc(0.01)
			d.innerRadius:SetConstantMessages(true)
			l:AddChild(d.innerRadius)
		l:Pop()
	l:Pop()

	d.newLayer = LM.GUI.CheckBox("Create star in new layer")
	l:AddChild(d.newLayer, LM.GUI.ALIGN_LEFT)

	return d
end

function LM_StarDialog:UpdateWidgets()
	self.numPoints:SetValue(LM_Star.numPoints)
	self.innerRadius:SetValue(LM_Star.innerRadius)
	self.newLayer:SetValue(LM_Star.newLayer)
	if (self.moho.layer:LayerType() == MOHO.LT_VECTOR) then
		self.newLayer:Enable(true)
	else
		self.newLayer:Enable(false)
	end
	self:UpdatePreview()
end

function LM_StarDialog:OnValidate()
	local b = true
	if (not self:Validate(self.numPoints, 3, 20)) then
		b = false
	end
	if (not self:Validate(self.innerRadius, 0, 0.75)) then
		b = false
	end
	return b
end

function LM_StarDialog:HandleMessage(what)
	if (what == self.UPDATE) then
		self:UpdatePreview()
	end
end

function LM_StarDialog:OnOK()
	LM_Star.numPoints = self.numPoints:IntValue()
	LM_Star.innerRadius = self.innerRadius:FloatValue()
	LM_Star.newLayer = self.newLayer:Value()
end

function LM_StarDialog:UpdatePreview()
	local mesh = self.preview:Mesh()
	mesh:Clear()

	local numPoints = self.numPoints:IntValue()
	numPoints = LM.Clamp(numPoints, 3, 20)

	local innerRadius = self.innerRadius:FloatValue()
	innerRadius = LM.Clamp(innerRadius, 0, 0.75)

	LM_Star:BuildStar(mesh, numPoints, innerRadius)
	self.preview:CreateShape(true)
	self.preview:Refresh()
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_Star:IsEnabled(moho)
	if (moho.layer:CurrentAction() ~= "") then
		return false -- creating new objects in the middle of an action can lead to unexpected results
	end
	return true
end

function LM_Star:Run(moho)
	local dlog = LM_StarDialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end

	if (LM_Star.newLayer or (moho.layer:LayerType() ~= MOHO.LT_VECTOR)) then
		moho.document:PrepUndo(nil)
		local layer = moho:CreateNewLayer(MOHO.LT_VECTOR)
		layer:SetName("Star")
	else
		moho.document:PrepUndo(moho.layer)
	end
	moho.document:SetDirty()

	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	self:BuildStar(mesh, self.numPoints, self.innerRadius)
	moho:CreateShape(true)
	mesh:AddGroup("Star")
end

function LM_Star:BuildStar(mesh, numPoints, innerRadius)
	local oldNumPoints = mesh:CountPoints()
	local v = LM.Vector2:new_local()
	local angle = math.pi / 2
	local dAngle = math.pi / numPoints
	local bigR = 0.75
	local littleR = innerRadius --0.2864745

	mesh:SelectNone()

	for i = 1, numPoints do
		v.x = bigR * math.cos(angle)
		v.y = bigR * math.sin(angle)
		if (i == 1) then
			mesh:AddLonePoint(v, 0)
		else
			mesh:AppendPoint(v, 0)
		end
		angle = angle + dAngle

		v.x = littleR * math.cos(angle)
		v.y = littleR * math.sin(angle)
		mesh:AppendPoint(v, 0)
		angle = angle + dAngle
	end

	v.x = bigR * math.cos(angle)
	v.y = bigR * math.sin(angle)
	mesh:AppendPoint(v, 0)

	mesh:WeldPoints(oldNumPoints, oldNumPoints + numPoints * 2, 0)

	for i = 0, numPoints * 2 - 1 do
		mesh:Point(oldNumPoints + i):SetCurvature(MOHO.PEAKED, 0)
	end

	mesh:SelectConnected()
end
