--[[
**************************************************
**************************************************

This script prompts the user for a text file.
This text file will be converted to text vectors in Moho,
according to the following procedure:

Each set of contiguous lines will appear as a single "screen". A blank
line in the text file separates each "screen" of credit text.
Each screen will appear on its own for the specified number of frames,
and then switched out for the next screen of credits. For example, if
the following were the contents of the credits file, the final
animation would show three screens of credits. The first screen would
have one line of text, the second would have three lines, and the third
would have two lines:

---- Begin text file ----
Moho 5.0

Produced by:
Lost Marble
2004

Programming:
Mike Clifton

**************************************************
**************************************************
]]

-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_Credits"

-- **************************************************
-- General information about this script
-- **************************************************

LM_Credits = {}

function LM_Credits:Name()
	return "Credits"
end

function LM_Credits:Version()
	return "5.0"
end

function LM_Credits:Description()
	return "Creates credit screens, based on a user-supplied text file."
end

function LM_Credits:Creator()
	return "Lost Marble"
end

function LM_Credits:UILabel()
	return("Credits...")
end

function LM_Credits:LoadPrefs(prefs)
	self.font = prefs:GetString("LM_Credits.font", "")
end

function LM_Credits:SavePrefs(prefs)
	prefs:SetString("LM_Credits.font", self.font)
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_Credits.duration = 24
LM_Credits.delay = 10
LM_Credits.fade = 3
LM_Credits.center = true
LM_Credits.font = ""

-- **************************************************
-- Credits dialog
-- **************************************************

local LM_CreditsDialog = {}

LM_CreditsDialog.UPDATE = MOHO.MSG_BASE

function LM_CreditsDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Credits", LM_CreditsDialog)
	local l = d:GetLayout()

	d.moho = moho

	l:PushH()
	l:PushV()

	l:AddChild(LM.GUI.StaticText("You will be prompted to select a text file that contains your credits."), LM.GUI.ALIGN_LEFT)
	l:AddChild(LM.GUI.StaticText("Try to keep your text file short, or this will take a long time."), LM.GUI.ALIGN_LEFT)
	l:AddChild(LM.GUI.StaticText("The color of the credit text will be based on the current fill color/style."), LM.GUI.ALIGN_LEFT)
	l:AddChild(LM.GUI.Divider(false), LM.GUI.ALIGN_FILL)

	l:PushH()
		l:PushV()
			l:AddChild(LM.GUI.StaticText("Duration (frames) for each credit screen"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Delay (frames) between credit screens"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Fade (frames)"), LM.GUI.ALIGN_LEFT)
		l:Pop()
		l:PushV()
			d.duration = LM.GUI.TextControl(0, "0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.duration)
			d.delay = LM.GUI.TextControl(0, "0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.delay)
			d.fade = LM.GUI.TextControl(0, "0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.fade)
		l:Pop()
	l:Pop()

	d.center = LM.GUI.CheckBox("Center text")
	l:AddChild(d.center, LM.GUI.ALIGN_LEFT)

	d.fontPreview = MOHO.FontPreview(64, 64)
	l:AddChild(d.fontPreview, LM.GUI.ALIGN_FILL)

	l:Pop()

	l:AddPadding(16)
	d.fontList = LM.GUI.TextList(200, 64, self.UPDATE)
	l:AddChild(d.fontList, LM.GUI.ALIGN_FILL)
	moho:FillInFontList(d.fontList)

	l:Pop()

	return d
end

function LM_CreditsDialog:UpdateWidgets()
	self.duration:SetValue(LM_Credits.duration)
	self.delay:SetValue(LM_Credits.delay)
	self.fade:SetValue(LM_Credits.fade)
	self.center:SetValue(LM_Credits.center)
	if (not self.fontList:SetSelItem(LM_Credits.font)) then
		self.fontList:SetSelItem(0)
	end
	self:HandleMessage(self.UPDATE)
end

function LM_CreditsDialog:OnValidate()
	local b = true
	if (not self:Validate(self.duration, 1, 1000000)) then
		b = false
	end
	if (not self:Validate(self.delay, 0, 1000000)) then
		b = false
	end
	if (not self:Validate(self.fade, 0, self.duration:Value() / 2)) then
		b = false
	end
	return b
end

function LM_CreditsDialog:HandleMessage(what)
	if (what == self.UPDATE) then
		self.fontPreview:SetPreviewText(self.fontList:SelItemLabel())
		self.fontPreview:SetFontName(self.fontList:SelItemLabel())
		self.fontPreview:Refresh()
	end
end

function LM_CreditsDialog:OnOK()
	LM_Credits.duration = self.duration:IntValue()
	LM_Credits.delay = self.delay:IntValue()
	LM_Credits.fade = self.fade:IntValue()
	LM_Credits.center = self.center:Value()
	LM_Credits.font = self.fontList:SelItemLabel()
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_Credits:Run(moho)
	local dlog = LM_CreditsDialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end

	local path = LM.GUI.OpenFile("Select Credits File")
	if (path == "") then
		return
	end

	local f = io.open(path, "r")
	if (f == nil) then
		return
	end

	moho.document:SetDirty()
	moho.document:PrepUndo(NULL)

	local page = 0
	local frame = moho.frame

	local group = moho:CreateNewLayer(MOHO.LT_GROUP)
	group:SetName("Credits")
	local scale = LM.Vector3:new_local()
	scale:Set(0.25, 0.25, 0.25)
	group.fScale:SetValue(0, scale)
	group.fScale:SetValue(moho.frame, scale)
	group.fVisibility:SetValue(0, false)
	group.fVisibility:SetValue(moho.frame, true)

	local line = f:read()
	while (line ~= nil) do
		local layer = moho:CreateNewLayer(MOHO.LT_VECTOR)
		layer:SetName(line)
		moho:PlaceLayerInGroup(layer, group, false)

		layer:ShowConstructionCurves(false)
		layer.fVisibility:SetValue(0, false)
		layer.fVisibility:SetValue(frame, true)
		layer.fVisibility:SetValue(frame + self.duration, false)

		local lineOffset = 1
		moho:InsertText(line, self.font, true, self.center, lineOffset)

		line = f:read()
		while (line ~= nil) do
			if (line == "") then
				break
			end
			lineOffset = lineOffset + 1
			moho:InsertText(line, self.font, true, self.center, lineOffset)
			if (lineOffset > 5) then
				break
			end
			line = f:read()
		end

		line = f:read()
		frame = frame + self.duration + self.delay

		page = page + 1
		if (page > 10) then
			break
		end
	end

	moho:SetSelLayer(group)

	f:close()
end
