-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_EnergyCloud"

-- **************************************************
-- General information about this script
-- **************************************************

LM_EnergyCloud = {}

function LM_EnergyCloud:Name()
	return "Energy Cloud"
end

function LM_EnergyCloud:Version()
	return "5.0"
end

function LM_EnergyCloud:Description()
	return "Creates an energy cloud effect."
end

function LM_EnergyCloud:Creator()
	return "Lost Marble"
end

function LM_EnergyCloud:UILabel()
	return("Energy Cloud...")
end

-- **************************************************
-- Energy Cloud dialog
-- **************************************************

local LM_EnergyCloudDialog = {}

function LM_EnergyCloudDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Energy Cloud", LM_EnergyCloudDialog)
	local l = d:GetLayout()

	d.moho = moho

	l:PushH()
		l:AddChild(LM.GUI.StaticText("Choose color:"))
		d.color = LM.GUI.ColorSwatch(false)
		l:AddChild(d.color)
	l:Pop()

	return d
end

function LM_EnergyCloudDialog:UpdateWidgets()
	self.color:SetValue(LM_EnergyCloud.color:AsColorStruct())
end

function LM_EnergyCloudDialog:OnOK()
	LM_EnergyCloud.color:Set(self.color:Value())
end

-- **************************************************
-- The guts of this script
-- **************************************************

LM_EnergyCloud.color = LM.ColorVector:new_local()
LM_EnergyCloud.color:Set(0.38, 0.9, 0.2)
LM_EnergyCloud.pCount = 75
LM_EnergyCloud.lifetime = 20
LM_EnergyCloud.shadowColor = LM.ColorVector:new_local()

function LM_EnergyCloud:Run(moho)
	local dlog = LM_EnergyCloudDialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end

	moho.document:PrepUndo(nil)
	moho.document:SetDirty()

	local v3 = LM.Vector3:new_local()
	local particles = moho:LayerAsParticle(moho:CreateNewLayer(MOHO.LT_PARTICLE))
	particles:SetName("Energy Cloud")

	particles:SetNumParticles(self.pCount, 20)
	particles:SetLifetime(self.lifetime)
	particles:SetOrientation(true)
	particles:SetFreeFloating(true)
	particles:SetEvenlySpaced(false)
	particles:SetRandomStartTime(false)
	particles:SetDirection(0, math.rad(180))
	v3:Set(0.2, 0.2, 0)
	particles:SetSourceDimensions(v3)
	particles:SetVelocity(0.01, 0)
	particles:SetDamping(0)
	particles:SetAcceleration(0, 0)
	particles:SetFullSpeedStart(true)

	for i = 1, 6 do
		self:AddParticle(moho, particles)
	end

	self.shadowColor:Set(self.color)
	self.shadowColor:Invert()
	self.shadowColor.a = 0.75
	particles.fLayerShadow:SetValue(0, true)
	particles.fShadowAngle:SetValue(0, 0)
	particles.fShadowOffset:SetValue(0, 0)
	particles.fShadowBlur:SetValue(0, 8 / 240)
	particles.fShadowExpansion:SetValue(0, 4 / 240)
	particles.fShadowColor:SetValue(0, self.shadowColor:AsColorStruct())
	particles.fBlur:SetValue(0, 8 / 240)

	particles:FinalizeSettings()
	particles:Expand(false)
	moho:SetSelLayer(particles)
end

function LM_EnergyCloud:AddParticle(moho, particles)
	-- Create a new vector layer and add it to the particle group
	local layer = moho:CreateNewLayer(MOHO.LT_VECTOR)
	moho:PlaceLayerInGroup(layer, particles)
	layer:SetName("p" .. particles:CountLayers())

	-- Create a little droplet shape
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local color = self.color * MOHO.RandomRange(0.9, 1.1)
	color.r = color.r * MOHO.RandomRange(0.95, 1.05)
	color.g = color.g * MOHO.RandomRange(0.95, 1.05)
	color.b = color.b * MOHO.RandomRange(0.95, 1.05)
	color.a = MOHO.RandomRange(0.1, 0.3)

	local radius = MOHO.RandomRange(0.1, 0.3)
	local blurRadius = MOHO.RandomRange(16, 20) / 240
	local v = LM.Vector2:new_local()

	mesh:SelectNone()
	v:Set(0, radius)
	mesh:AddLonePoint(v, 0)
	v:Set(-radius, 0)
	mesh:AppendPoint(v, 0)
	v:Set(0, -radius)
	mesh:AppendPoint(v, 0)
	v:Set(radius, 0)
	mesh:AppendPoint(v, 0)
	mesh:AppendPoint(v, 0)
	mesh:WeldPoints(4, 0, 0)

	-- Now add a shape and set its color and style properties
	mesh:SelectConnected()
	local id = moho:CreateShape(true, 0, false)
	if (id >= 0) then
		local shape = mesh:Shape(id)
		shape.fHasOutline = false
		shape:RemoveStyles()
		shape.fMyStyle:SetSoftEdge(blurRadius)
		shape.fMyStyle.fFillCol:SetValue(0, color:AsColorStruct())
	end

	mesh:SelectNone()
	radius = radius - radius / 4
	v:Set(0, radius)
	mesh:AddLonePoint(v, 0)
	v:Set(-radius, 0)
	mesh:AppendPoint(v, 0)
	v:Set(0, -radius)
	mesh:AppendPoint(v, 0)
	v:Set(radius, 0)
	mesh:AppendPoint(v, 0)
	mesh:AppendPoint(v, 0)
	mesh:WeldPoints(8, 4, 0)

	-- Add a second shape in the center of the first, without blur
	mesh:SelectConnected()
	local id = moho:CreateShape(true, 0, false)
	if (id >= 0) then
		local shape = mesh:Shape(id)
		shape.fHasOutline = false
		shape:RemoveStyles()
		shape.fMyStyle:SetShading(math.rad(135), 3 / 240, 3 / 240, self.shadowColor:AsColorStruct())
		shape.fMyStyle.fFillCol:SetValue(0, color:AsColorStruct())
	end

	local v3 = LM.Vector3:new_local()
	v3:Set(1, 1, 1)
	layer.fScale:SetValue(0, v3)
	v3:Set(0, 0, 0)
	layer.fScale:SetValue(self.lifetime, v3)

	radius = MOHO.RandomRange(0.2, 0.6)
	local angle = math.rad(MOHO.RandomRange(-360, 360))

	v3:Set(radius / 3, 0, 0)
	v3:Rotate(LM.Z_AXIS, angle / 3)
	layer.fTranslation:SetValue(self.lifetime / 3, v3)

	v3:Set(2 * radius / 3, 0, 0)
	v3:Rotate(LM.Z_AXIS, 2 * angle / 3)
	layer.fTranslation:SetValue(2 * self.lifetime / 3, v3)

	v3:Set(radius, 0, 0)
	v3:Rotate(LM.Z_AXIS, angle)
	layer.fTranslation:SetValue(self.lifetime, v3)
end
