-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_Explosion"

-- **************************************************
-- General information about this script
-- **************************************************

LM_Explosion = {}

function LM_Explosion:Name()
	return "Explosion"
end

function LM_Explosion:Version()
	return "5.0"
end

function LM_Explosion:Description()
	return "Creates an explosion starting at the current frame."
end

function LM_Explosion:Creator()
	return "Lost Marble"
end

function LM_Explosion:UILabel()
	return("Explosion...")
end

-- **************************************************
-- Explosion dialog
-- **************************************************

local LM_ExplosionDialog = {}

function LM_ExplosionDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Explosion", LM_ExplosionDialog)
	local l = d:GetLayout()

	d.moho = moho

	l:PushH()
		l:AddChild(LM.GUI.StaticText("Direction"))
		d.angle = LM.GUI.AngleWidget()
		l:AddChild(d.angle)
	l:Pop()

	l:PushH()
		l:AddChild(LM.GUI.StaticText("Spread"))
		d.spread = LM.GUI.TextControl(0, "0000", 0, LM.GUI.FIELD_UFLOAT)
		l:AddChild(d.spread)
	l:Pop()

	return d
end

function LM_ExplosionDialog:UpdateWidgets()
	self.angle:SetValue(LM_Explosion.angle)
	self.spread:SetValue(math.deg(LM_Explosion.spread) * 2)
end

function LM_ExplosionDialog:OnValidate()
	local b = true
	if (not self:Validate(self.spread, 0, 360)) then
		b = false
	end
	return b
end

function LM_ExplosionDialog:OnOK()
	LM_Explosion.angle = self.angle:Value()
	LM_Explosion.spread = math.rad(self.spread:FloatValue()) / 2
end

-- **************************************************
-- The guts of this script
-- **************************************************

LM_Explosion.angle = math.rad(90)
LM_Explosion.spread = math.rad(80)
LM_Explosion.color = LM.ColorVector:new_local()
LM_Explosion.color:Set(0.5, 0.5, 0.5)
LM_Explosion.pCount = 2000
LM_Explosion.lifetime = 30
LM_Explosion.flameDuration = 8
LM_Explosion.flameColor = LM.ColorVector:new_local()
LM_Explosion.flameColor:Set(0.3, 0.15, 0.08)
LM_Explosion.flameColor = LM_Explosion.flameColor * 3
LM_Explosion.extraShading = true
LM_Explosion.darkSmokeColor = LM.ColorVector:new_local()
LM_Explosion.darkSmokeColor:Set(0.1, 0.1, 0.1)

function LM_Explosion:Run(moho)
	local dlog = LM_ExplosionDialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end

	moho.document:PrepUndo(nil)
	moho.document:SetDirty()

	if (self.flameDuration > 0) then -- create two particle systems: smoke and flame
		local group = moho:LayerAsGroup(moho:CreateNewLayer(MOHO.LT_GROUP))
		group:SetName("Explosion")
		local smokeLayer = self:CreateSmoke(moho)
		moho:PlaceLayerInGroup(smokeLayer, group)
		moho:SetSelLayer(group)
		local flameLayer = self:CreateFlame(moho)
		moho:PlaceLayerInGroup(flameLayer, group)
		group:Expand(false)
		moho:SetSelLayer(group)
	else -- just the basic smoke
		local smokeLayer = self:CreateSmoke(moho)
		moho:SetSelLayer(smokeLayer)
	end
end

function LM_Explosion:CreateSmoke(moho)
	local v3 = LM.Vector3:new_local()
	local particles = moho:LayerAsParticle(moho:CreateNewLayer(MOHO.LT_PARTICLE))
	particles:SetName("Smoke")

	particles:SetNumParticles(self.pCount, 20)
	particles:SetLifetime(self.lifetime)
	particles:SetOrientation(false)
	particles:SetFreeFloating(true)
	particles:SetEvenlySpaced(false)
	particles:SetRandomStartTime(false)
	particles:SetDirection(self.angle, self.spread)
	v3:Set(0.4, 0.4, 0.4)
	particles:SetSourceDimensions(v3)
	particles:SetVelocity(3, 1)
	particles:SetDamping(2)
	particles:SetAcceleration(0, 1)
	particles:SetFullSpeedStart(false)
	particles:RunningTrack():SetValue(0, false)
	particles:RunningTrack():SetValue(moho.frame + 1, true)
	particles:RunningTrack():SetValue(moho.frame + 1 + self.flameDuration + 1, false)

	for i = 1, 12 do
		self:AddSmokeParticle(moho, particles)
	end

	particles:FinalizeSettings()
	particles:Expand(false)
	return particles
end

function LM_Explosion:CreateFlame(moho)
	local v3 = LM.Vector3:new_local()
	local particles = moho:LayerAsParticle(moho:CreateNewLayer(MOHO.LT_PARTICLE))
	particles:SetName("Flame")

	particles:SetNumParticles(self.pCount, 20)
	particles:SetLifetime(self.flameDuration)
	particles:SetOrientation(false)
	particles:SetFreeFloating(true)
	particles:SetEvenlySpaced(false)
	particles:SetRandomStartTime(false)
	particles:SetDirection(self.angle, self.spread)
	v3:Set(0.4, 0.4, 0.4)
	particles:SetSourceDimensions(v3 * 0.75)
	particles:SetVelocity(3, 1)
	particles:SetDamping(2)
	particles:SetAcceleration(0, 1)
	particles:SetFullSpeedStart(false)
	particles:RunningTrack():SetValue(0, false)
	particles:RunningTrack():SetValue(moho.frame + 1, true)
	particles:RunningTrack():SetValue(moho.frame + 1 + self.flameDuration, false)

	for i = 1, 6 do
		self:AddFlameParticle(moho, particles)
	end

	particles:FinalizeSettings()
	particles:Expand(false)
	return particles
end

function LM_Explosion:AddSmokeParticle(moho, particles)
	-- Create a new vector layer and add it to the particle group
	local layer = moho:CreateNewLayer(MOHO.LT_VECTOR)
	moho:PlaceLayerInGroup(layer, particles)
	layer:SetName("p" .. particles:CountLayers())

	-- Create a little blob shape
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local numPoints = 4
	local n = mesh:CountPoints()
	local v = LM.Vector2:new_local()
	local angle = math.pi / 2
	local dAngle = math.pi / numPoints
	local radius = MOHO.RandomRange(0.003, 0.15)
	local flameDuration = 0

	if (self.flameDuration > 0) then
		flameDuration = self.flameDuration + 1
	end

	mesh:SelectNone()
	for i = 1, numPoints do
		local r = radius * MOHO.RandomRange(0.25, 1)
		v.x = r * math.cos(angle)
		v.y = r * math.sin(angle)
		if (i == 1) then
			mesh:AddLonePoint(v, 0)
		else
			mesh:AppendPoint(v, 0)
		end
		angle = angle + dAngle

		r = radius * MOHO.RandomRange(0.25, 1)
		v.x = r * math.cos(angle)
		v.y = r * math.sin(angle)
		mesh:AppendPoint(v, 0)
		angle = angle + dAngle
	end
	mesh:AppendPoint(v, 0)
	mesh:WeldPoints(n + numPoints * 2, n, 0)

	-- Now add a shape and set its color and style properties
	local smokeStartFrame = 0
	mesh:SelectConnected()
	local id = moho:CreateShape(true, 0, false)
	if (id >= 0) then
		local shape = mesh:Shape(id)
		shape.fHasOutline = false
		shape:RemoveStyles()
		shape.fMyStyle:SetSoftEdge(MOHO.RandomRange(12, 16) / 240)

		local shadingColor = self.color * MOHO.RandomRange(0.5, 1.5)
		shadingColor.r = shadingColor.r * MOHO.RandomRange(0.9, 1.1)
		shadingColor.g = shadingColor.g * MOHO.RandomRange(0.9, 1.1)
		shadingColor.b = shadingColor.b * MOHO.RandomRange(0.9, 1.1)
		shadingColor:Invert()
		shadingColor.a = 0.5
		if (self.extraShading) then
			shape.fMyStyle:SetShading(math.rad(135), 3 / 240, 3 / 240, shadingColor:AsColorStruct())
		end

		if (flameDuration > 0) then -- apply a flame effect to begin the particle's lifetime
			smokeStartFrame = flameDuration
			local color = self.darkSmokeColor * MOHO.RandomRange(0.9, 1.1)
			color.r = color.r * MOHO.RandomRange(0.95, 1.05)
			color.g = color.g * MOHO.RandomRange(0.95, 1.05)
			color.b = color.b * MOHO.RandomRange(0.95, 1.05)
			color.a = MOHO.RandomRange(0.7, 0.8)
			shape.fMyStyle.fFillCol:SetValue(0, color:AsColorStruct())
			shape.fMyStyle.fFillCol:SetValue(smokeStartFrame - flameDuration / 4, color:AsColorStruct())
		end

		local color = self.color * MOHO.RandomRange(0.5, 1.5)
		color.r = color.r * MOHO.RandomRange(0.95, 1.05)
		color.g = color.g * MOHO.RandomRange(0.95, 1.05)
		color.b = color.b * MOHO.RandomRange(0.95, 1.05)
		color.a = MOHO.RandomRange(0.1, 0.35)
		shape.fMyStyle.fFillCol:SetValue(smokeStartFrame, color:AsColorStruct())
		color = self.color * MOHO.RandomRange(0.5, 1.5)
		color.r = color.r * MOHO.RandomRange(0.95, 1.05)
		color.g = color.g * MOHO.RandomRange(0.95, 1.05)
		color.b = color.b * MOHO.RandomRange(0.95, 1.05)
		color.a = MOHO.RandomRange(0.1, 0.35)
		shape.fMyStyle.fFillCol:SetValue(self.lifetime / 2, color:AsColorStruct())
		color = self.color * MOHO.RandomRange(0.5, 1.5)
		color.r = color.r * MOHO.RandomRange(0.95, 1.05)
		color.g = color.g * MOHO.RandomRange(0.95, 1.05)
		color.b = color.b * MOHO.RandomRange(0.95, 1.05)
		color.a = 0
		shape.fMyStyle.fFillCol:SetValue(self.lifetime, color:AsColorStruct())
	end

	-- Scale this layer to make it appear to grow and then fade out
	local scaleVec = LM.Vector3:new_local()
	scaleVec:Set(0.25, 0.25, 0.25)
	layer.fScale:SetValue(0, scaleVec)
	scaleVec:Set(2, 2, 2)
	layer.fScale:SetValue(self.lifetime / 2, scaleVec)
	scaleVec:Set(0.1, 0.1, 0.1)
	layer.fScale:SetValue(self.lifetime, scaleVec)
	if (flameDuration > 0) then
		layer.fScale:SetValue(smokeStartFrame, layer.fScale:GetValue(smokeStartFrame))
		scaleVec:Set(0.75, 0.75, 0.75)
		layer.fScale:SetValue(0, scaleVec)
	end
end

function LM_Explosion:AddFlameParticle(moho, particles)
	-- Create a new vector layer and add it to the particle group
	local layer = moho:CreateNewLayer(MOHO.LT_VECTOR)
	moho:PlaceLayerInGroup(layer, particles)
	layer:SetName("p" .. particles:CountLayers())

	-- Create a little blob shape
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local numPoints = 4
	local n = mesh:CountPoints()
	local v = LM.Vector2:new_local()
	local angle = math.pi / 2
	local dAngle = math.pi / numPoints
	local radius = MOHO.RandomRange(0.03, 0.15)

	mesh:SelectNone()
	for i = 1, numPoints do
		local r = radius * MOHO.RandomRange(0.25, 1)
		v.x = r * math.cos(angle)
		v.y = r * math.sin(angle)
		if (i == 1) then
			mesh:AddLonePoint(v, 0)
		else
			mesh:AppendPoint(v, 0)
		end
		angle = angle + dAngle

		r = radius * MOHO.RandomRange(0.25, 1)
		v.x = r * math.cos(angle)
		v.y = r * math.sin(angle)
		mesh:AppendPoint(v, 0)
		angle = angle + dAngle
	end
	mesh:AppendPoint(v, 0)
	mesh:WeldPoints(n + numPoints * 2, n, 0)

	-- Now add a shape and set its color and style properties
	mesh:SelectConnected()
	local id = moho:CreateShape(true, 0, false)
	if (id >= 0) then
		local shape = mesh:Shape(id)
		shape.fHasOutline = false
		shape:RemoveStyles()
		shape.fMyStyle:SetSoftEdge(MOHO.RandomRange(6, 12) / 240)

		local color = self.flameColor * MOHO.RandomRange(0.5, 1.5)
		color.r = color.r * MOHO.RandomRange(0.95, 1.05)
		color.g = color.g * MOHO.RandomRange(0.95, 1.05)
		color.b = color.b * MOHO.RandomRange(0.95, 1.05)
		color.a = MOHO.RandomRange(0.1, 0.35)
		shape.fMyStyle.fFillCol:SetValue(0, color:AsColorStruct())
		color = self.flameColor * MOHO.RandomRange(0.5, 1.5)
		color.r = color.r * MOHO.RandomRange(0.95, 1.05)
		color.g = color.g * MOHO.RandomRange(0.95, 1.05)
		color.b = color.b * MOHO.RandomRange(0.95, 1.05)
		color.a = MOHO.RandomRange(0.1, 0.35)
		shape.fMyStyle.fFillCol:SetValue(self.flameDuration / 2, color:AsColorStruct())
		color = self.flameColor * MOHO.RandomRange(0.5, 1.5)
		color.r = color.r * MOHO.RandomRange(0.95, 1.05)
		color.g = color.g * MOHO.RandomRange(0.95, 1.05)
		color.b = color.b * MOHO.RandomRange(0.95, 1.05)
		color.a = 0
		shape.fMyStyle.fFillCol:SetValue(self.flameDuration + 1, color:AsColorStruct())
	end

	-- Scale this layer to make it appear to grow and then fade out
	local scaleVec = LM.Vector3:new_local()
	scaleVec:Set(0.5, 0.5, 0.5)
	layer.fScale:SetValue(0, scaleVec)
	scaleVec:Set(2.5, 2.5, 2.5)
	layer.fScale:SetValue(self.lifetime / 2, scaleVec)
	scaleVec:Set(3, 3, 3)
	layer.fScale:SetValue(self.lifetime, scaleVec)
	layer:SetBlendingMode(MOHO.BM_ADD)
end
