-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_AddBone"

-- **************************************************
-- General information about this script
-- **************************************************

LM_AddBone = {}

function LM_AddBone:Name()
	return "Add Bone"
end

function LM_AddBone:Version()
	return "5.0"
end

function LM_AddBone:Description()
	return "Add a new bone (hold <shift> to constrain, <alt> to select a bone)"
end

function LM_AddBone:Creator()
	return "Lost Marble"
end

function LM_AddBone:UILabel()
	return("Add Bone")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_AddBone.selecting = false
LM_AddBone.startVec = LM.Vector2:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_AddBone:IsEnabled(moho)
	if (moho.frame > 0) then
		return false
	end
	local skel = moho:Skeleton()
	if (skel == nil) then
		return false
	end
	if (moho.layer:CurrentAction() ~= "") then
		return false -- creating new objects in the middle of an action can lead to unexpected results
	end
	if (moho:CountSelectedBones() > 1) then
		return false -- undefined in this case which bone should be the parent
	end
	return true
end

function LM_AddBone:OnMouseDown(moho, mouseEvent)
	if (mouseEvent.altKey) then
		self.selecting = true
		LM_SelectBone:Select(moho, mouseEvent.pt, mouseEvent.vec, mouseEvent.view, false, false)
		return
	end
	self.selecting = false

	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	-- find the parent bone (if any), and deselect it
	local parentID = -1
	for i = 0, skel:CountBones() - 1 do
		local bone = skel:Bone(i)
		if (bone.fSelected) then
			parentID = i
			bone.fSelected = false
			break
		end
	end

	local bone = skel:AddBone(moho.frame)
	bone.fParent = parentID
	if (moho.gridOn) then
		moho:SnapToGrid(mouseEvent.vec)
	end
	bone.fAnimPos:SetValue(moho.frame, mouseEvent.vec)

	if (parentID >= 0) then
		-- transform the bone's base point into parent's coordinate system
		local inverseM = LM.Matrix:new_local()
		inverseM:Set(skel:Bone(parentID).fRestMatrix)
		inverseM:Invert()
		local pos = bone.fAnimPos:GetValue(moho.frame)
		inverseM:Transform(pos)
		bone.fAnimPos:SetValue(moho.frame, pos)
	end

	bone.fLength = 0.0001 -- theoretically, this would be zero, but we don't really like zero-length bones
	bone.fAnimAngle:SetValue(moho.frame, 0)
	bone.fPos:Set(bone.fAnimPos:GetValue(moho.frame))
	bone.fTempPos:Set(bone.fPos)
	bone.fTempLength = bone.fLength
	bone.fAngle = bone.fAnimAngle:GetValue(moho.frame)
	bone.fTempAngle = bone.fAngle
	bone.fSelected = true
	LM_TranslateBone.selID = skel:CountBones() - 1
	LM_TranslateBone.boneEnd = 1
	LM_TranslateBone:OnMouseDown(moho, mouseEvent)
	moho:UpdateSelectedChannels()
end

function LM_AddBone:OnMouseMoved(moho, mouseEvent)
	if (self.selecting) then
		return
	end
	LM_TranslateBone:OnMouseMoved(moho, mouseEvent)
end

function LM_AddBone:OnMouseUp(moho, mouseEvent)
	if (self.selecting) then
		return
	end
	LM_TranslateBone:OnMouseUp(moho, mouseEvent)
end

function LM_AddBone:OnKeyDown(moho, keyEvent)
	LM_SelectBone:OnKeyDown(moho, keyEvent)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_AddBone.CHANGE = MOHO.MSG_BASE
LM_AddBone.DUMMY = MOHO.MSG_BASE + 1
LM_AddBone.SELECTITEM = MOHO.MSG_BASE + 2

function LM_AddBone:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu("Select Bone")

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	self.boneName = LM.GUI.TextControl(0, "Room For a Big Long Name", self.CHANGE, LM.GUI.FIELD_TEXT)
	self.boneName:SetValue("")
	layout:AddChild(self.boneName)
end

function LM_AddBone:UpdateWidgets(moho)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	local selID = skel:SelectedBoneID()

	MOHO.BuildBoneMenu(self.menu, skel, self.SELECTITEM, self.DUMMY)

	if (selID >= 0) then
		self.boneName:SetValue(skel:Bone(selID):Name())
	else
		self.boneName:SetValue("")
	end
end

function LM_AddBone:HandleMessage(moho, view, msg)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	if (msg == self.CHANGE) then
		for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			if (bone.fSelected) then
				moho.document:PrepUndo(moho.layer)
				moho.document:SetDirty()
				bone:SetName(self.boneName:Value())
				break
			end
		end
	elseif (msg >= self.SELECTITEM) then
		for i = 0, skel:CountBones() - 1 do
			skel:Bone(i).fSelected = (i == msg - self.SELECTITEM)
		end
		moho:UpdateUI()
	end
end
