-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_AddPoint"

-- **************************************************
-- General information about this script
-- **************************************************

LM_AddPoint = {}

function LM_AddPoint:Name()
	return "Add Point"
end

function LM_AddPoint:Version()
	return "5.0"
end

function LM_AddPoint:Description()
	return "Add a point (press <space> to weld, hold <alt> to disable auto-welding)"
end

function LM_AddPoint:Creator()
	return "Lost Marble"
end

function LM_AddPoint:UILabel()
	return("Add Point")
end

function LM_AddPoint:LoadPrefs(prefs)
	self.sharpCorners = prefs:GetBool("LM_AddPoint.sharpCorners", false)
end

function LM_AddPoint:SavePrefs(prefs)
	prefs:SetBool("LM_AddPoint.sharpCorners", self.sharpCorners)
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_AddPoint.numSel = 0
LM_AddPoint.selID = 0

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_AddPoint:IsEnabled(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return false
	end
	if (moho.layer:CurrentAction() ~= "") then
		return false -- creating new objects in the middle of an action can lead to unexpected results
	end
	return true
end

function LM_AddPoint:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	local closestID = -1
	local closest = 1e6
	local curveID = -1
	local segID = -1

	mesh:SelectNone()
	if (not(mouseEvent.altKey)) then
		-- find the closest point for auto-welding the new point to
		local testVec1 = LM.Vector2:new_local()
		testVec1:Set(mouseEvent.vec)
		closestID = mesh:ClosestPoint(testVec1)
		if (closestID >= 0) then
			local testVec2 = LM.Vector2:new_local()
			testVec2:Set(mesh:Point(closestID).fPos)

			-- convert the two chosen points to pixel coordinates
			-- this is to make sure that they are actually close enough in screen space
			local p1 = LM.Point:new_local()
			local p2 = LM.Point:new_local()
			local m = LM.Matrix:new_local()
			moho.layer:GetFullTransform(moho.frame, m, moho.document)
			m:Transform(testVec1)
			m:Transform(testVec2)
			mouseEvent.view:Graphics():WorldToScreen(testVec1, p1)
			mouseEvent.view:Graphics():WorldToScreen(testVec2, p2)
			p1.x = p1.x - p2.x
			p1.y = p1.y - p2.y
			closest = (p1.x * p1.x) + (p1.y * p1.y)
		end
	end

	if (closest < 100 and not(mouseEvent.altKey)) then -- add as an extension to an existing point
		if (mesh:Point(closestID):IsEndpoint()) then -- extend this curve
			mesh:AddPoint(mouseEvent.vec, closestID, moho.frame)
		else -- start a new curve that is connected to this point
			mesh:AddPoint(mouseEvent.vec, -1, moho.frame)
			if (self.sharpCorners) then
				mesh:Point(closestID):SetCurvature(MOHO.PEAKED, 0)
			end
			mesh:WeldPoints(mesh:CountPoints() - 2, closestID, moho.frame)
		end
		if (self.sharpCorners) then
			mesh:Point(mesh:CountPoints() - 1):SetCurvature(MOHO.PEAKED, 0)
		end
	else
		local curveID = -1
		local segID = -1

		if (not mouseEvent.altKey) then
			curveID, segID = mouseEvent.view:PickEdge(mouseEvent.pt, curveID, segID)
		end
		if (curveID >= 0) then -- add a point in the middle of some curve
			if (moho.gridOn) then
				moho:SnapToGrid(mouseEvent.vec)
			end
			mesh:AddPoint(mouseEvent.vec, curveID, segID, moho.frame)
		else -- add a new curve segment
			if (moho.gridOn) then
				moho:SnapToGrid(mouseEvent.vec)
			end
			mesh:AddPoint(mouseEvent.vec, -1, moho.frame)
			if (self.sharpCorners) then
				mesh:Point(mesh:CountPoints() - 2):SetCurvature(MOHO.PEAKED, 0)
			end
		end
		if (self.sharpCorners) then
			mesh:Point(mesh:CountPoints() - 1):SetCurvature(MOHO.PEAKED, 0)
		end
	end

	LM_TranslatePoints.selID = mesh:CountPoints() - 1
	LM_TranslatePoints:OnMouseDown(moho, mouseEvent)

	mouseEvent.view:DrawMe()
end

function LM_AddPoint:OnMouseMoved(moho, mouseEvent)
	LM_TranslatePoints:OnMouseMoved(moho, mouseEvent)
end

function LM_AddPoint:OnMouseUp(moho, mouseEvent)
	LM_TranslatePoints:OnMouseUp(moho, mouseEvent)
end

function LM_AddPoint:OnKeyDown(moho, keyEvent)
	LM_TranslatePoints:OnKeyDown(moho, keyEvent)
end

function LM_AddPoint:DrawMe(moho, view)
	LM_TranslatePoints:DrawMe(moho, view)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_AddPoint.DUMMY = MOHO.MSG_BASE
LM_AddPoint.AUTOWELD = MOHO.MSG_BASE + 1
LM_AddPoint.AUTOFILL = MOHO.MSG_BASE + 2
LM_AddPoint.SHARPCORNERS = MOHO.MSG_BASE + 3
LM_AddPoint.SELECTITEM = MOHO.MSG_BASE + 4
LM_AddPoint.sharpCorners = false

function LM_AddPoint:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu("Select Group")

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	self.autoWeldCheck = LM.GUI.CheckBox("Auto-weld", self.AUTOWELD)
	layout:AddChild(self.autoWeldCheck)

	self.autoFillCheck = LM.GUI.CheckBox("Auto-fill", self.AUTOFILL)
	layout:AddChild(self.autoFillCheck)

	self.sharpCornersCheck = LM.GUI.CheckBox("Sharp corners", self.SHARPCORNERS)
	layout:AddChild(self.sharpCornersCheck)
end

function LM_AddPoint:UpdateWidgets(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	MOHO.BuildGroupMenu(self.menu, mesh, self.SELECTITEM, self.DUMMY)

	self.autoWeldCheck:SetValue(LM_TranslatePoints.autoWeld)
	self.autoFillCheck:SetValue(LM_TranslatePoints.autoFill)
	self.sharpCornersCheck:SetValue(LM_AddPoint.sharpCorners)
end

function LM_AddPoint:HandleMessage(moho, view, msg)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (msg == self.AUTOWELD) then
		LM_TranslatePoints.autoWeld = self.autoWeldCheck:Value()
	elseif (msg == self.AUTOFILL) then
		LM_TranslatePoints.autoFill = self.autoFillCheck:Value()
	elseif (msg == self.SHARPCORNERS) then
		LM_AddPoint.sharpCorners = self.sharpCornersCheck:Value()
	elseif (msg >= self.SELECTITEM) then
		mesh:SelectNone()
		local i = msg - self.SELECTITEM
		local name = mesh:Group(i):Name()
		mesh:SelectGroup(name)
		moho:UpdateUI()
	end
end
