-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_CreateShape"

-- **************************************************
-- General information about this script
-- **************************************************

LM_CreateShape = {}

function LM_CreateShape:Name()
	return "Create Shape"
end

function LM_CreateShape:Version()
	return "5.0"
end

function LM_CreateShape:Description()
	return "Select edges to define a shape (press <space> to create shape, hold <shift> to modify current selection, hold <ctrl> to toggle lasso mode)"
end

function LM_CreateShape:Creator()
	return "Lost Marble"
end

function LM_CreateShape:UILabel()
	return("Create Shape")
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_CreateShape:IsEnabled(moho)
	if (moho.layer:CurrentAction() ~= "") then
		return false -- creating new objects in the middle of an action can lead to unexpected results
	end
	if (moho:CountEdges() > 0) then
		return true
	end
	return false
end

function LM_CreateShape:OnMouseDown(moho, mouseEvent)
	LM_SelectPoints.allowShapePicking = false
	LM_SelectPoints:OnMouseDown(moho, mouseEvent)
end

function LM_CreateShape:OnMouseMoved(moho, mouseEvent)
	LM_SelectPoints:OnMouseMoved(moho, mouseEvent)
end

function LM_CreateShape:OnMouseUp(moho, mouseEvent)
	LM_SelectPoints:OnMouseUp(moho, mouseEvent)
end

function LM_CreateShape:OnKeyDown(moho, keyEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (keyEvent.key == ' ') then
		self:HandleMessage(moho, keyEvent.view, self.CREATE)
	elseif (keyEvent.keyCode == LM.GUI.KEY_UP) then
		if (keyEvent.shiftKey) then -- raise shape to top
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected) then
					mesh:RaiseShape(i, true)
					break
				end
			end
		else -- raise shape
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected) then
					mesh:RaiseShape(i, false)
					break
				end
			end
		end
		keyEvent.view:DrawMe()
	elseif (keyEvent.keyCode == LM.GUI.KEY_DOWN) then
		if (keyEvent.shiftKey) then -- lower shape to bottom
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected) then
					mesh:LowerShape(i, true)
					break
				end
			end
		else -- lower shape
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected) then
					mesh:LowerShape(i, false)
					break
				end
			end
		end
		keyEvent.view:DrawMe()
	else
		LM_SelectShape:OnKeyDown(moho, keyEvent)
	end
end

function LM_CreateShape:DrawMe(moho, view)
	local g = view:Graphics()
	local matrix = LM.Matrix:new_local()

	moho.layer:GetFullTransform(moho.frame, matrix, moho.document)
	g:Push()
	g:ApplyMatrix(matrix)

	view:DrawPreviewShape()

	g:Pop()
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_CreateShape.CREATE = MOHO.MSG_BASE
LM_CreateShape.LASSO = MOHO.MSG_BASE + 1
LM_CreateShape.DUMMY = MOHO.MSG_BASE + 2
LM_CreateShape.SELECTITEM = MOHO.MSG_BASE + 3

function LM_CreateShape:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu("Select Group")

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	self.createBut = LM.GUI.Button("Create Shape", self.CREATE)
	layout:AddChild(self.createBut)

	self.lassoCheck = LM.GUI.CheckBox("Lasso mode", self.LASSO)
	layout:AddChild(self.lassoCheck)
end

function LM_CreateShape:UpdateWidgets(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	MOHO.BuildGroupMenu(self.menu, mesh, self.SELECTITEM, self.DUMMY)

	self.createBut:Enable(moho:CountSelectedEdges() > 0)

	self.lassoCheck:SetValue(LM_SelectPoints.lassoMode)
end

function LM_CreateShape:HandleMessage(moho, view, msg)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (msg == self.CREATE) then
		if (moho:CountSelectedEdges() > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			local shapeID = moho:CreateShape(true) -- filled shape
			if (shapeID < 0) then
				shapeID = moho:CreateShape(false) -- just an outline
				if (shapeID >= 0) then
					mesh:SelectNone()
					mesh:Shape(shapeID).fFillAllowed = false
				end
			end
			if (shapeID >= 0) then
				mesh:SelectNone()
				mesh:Shape(shapeID).fSelected = true
				moho:NewKeyframe(CHANNEL_FILL)
				moho:NewKeyframe(CHANNEL_LINE)
				moho:UpdateSelectedChannels()
			end
			view:DrawMe()
		end
	elseif (msg == self.LASSO) then
		LM_SelectPoints.lassoMode = self.lassoCheck:Value()
	elseif (msg >= self.SELECTITEM) then
		mesh:SelectNone()
		local i = msg - self.SELECTITEM
		local name = mesh:Group(i):Name()
		mesh:SelectGroup(name)
		moho:UpdateUI()
	end
end
