-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_Freehand"

-- **************************************************
-- General information about this script
-- **************************************************

LM_Freehand = {}

function LM_Freehand:Name()
	return "Freehand"
end

function LM_Freehand:Version()
	return "5.0"
end

function LM_Freehand:Description()
	return "Freehand drawing (hold <alt> to disable auto-welding)"
end

function LM_Freehand:Creator()
	return "Lost Marble"
end

function LM_Freehand:UILabel()
	return("Freehand")
end

function LM_Freehand:LoadPrefs(prefs)
	self.pixelTolerance = prefs:GetInt("LM_Freehand.pixelTolerance", 10)
	self.angleTolerance = prefs:GetInt("LM_Freehand.angleTolerance", 10)
	self.autoClose = prefs:GetBool("LM_Freehand.autoClose", false)
	self.autoFill = prefs:GetBool("LM_Freehand.autoFill", false)
	self.autoOutline = prefs:GetBool("LM_Freehand.autoOutline", true)
	self.widthMode = prefs:GetInt("LM_Freehand.widthMode", 1)
	self.minWidth = prefs:GetFloat("LM_Freehand.minWidth", 1)
	self.maxWidth = prefs:GetFloat("LM_Freehand.maxWidth", 8)
	self.taperStart = prefs:GetBool("LM_Freehand.taperStart", false)
	self.taperEnd = prefs:GetBool("LM_Freehand.taperEnd", false)
end

function LM_Freehand:SavePrefs(prefs)
	prefs:SetInt("LM_Freehand.pixelTolerance", self.pixelTolerance)
	prefs:SetInt("LM_Freehand.angleTolerance", self.angleTolerance)
	prefs:SetBool("LM_Freehand.autoClose", self.autoClose)
	prefs:SetBool("LM_Freehand.autoFill", self.autoFill)
	prefs:SetBool("LM_Freehand.autoOutline", self.autoOutline)
	prefs:SetInt("LM_Freehand.widthMode", self.widthMode)
	prefs:SetFloat("LM_Freehand.minWidth", self.minWidth)
	prefs:SetFloat("LM_Freehand.maxWidth", self.maxWidth)
	prefs:SetBool("LM_Freehand.taperStart", self.taperStart)
	prefs:SetBool("LM_Freehand.taperEnd", self.taperEnd)
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_Freehand.startingPoint = 0
LM_Freehand.previousX = 0
LM_Freehand.previousY = 0
LM_Freehand.previousDrawX = 0
LM_Freehand.previousDrawY = 0
LM_Freehand.previewWidth = 1
LM_Freehand.bindStartTo = -1
LM_Freehand.disableTaperStart = false
LM_Freehand.disableTaperEnd = false

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_Freehand:IsEnabled(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return false
	end
	if (moho.layer:CurrentAction() ~= "") then
		return false -- creating new objects in the middle of an action can lead to unexpected results
	end
	return true
end

function LM_Freehand:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	mesh:SelectNone()
	self.startingPoint = mesh:CountPoints()

	local closestID = -1
	local curveID = -1
	local segID = -1

	if (not(mouseEvent.altKey) and LM_TranslatePoints.autoWeld) then
		-- find the closest point for auto-welding the freehand curve to
		local testVec1 = LM.Vector2:new_local()
		testVec1:Set(mouseEvent.vec)
		closestID = mesh:ClosestPoint(testVec1)
		if (closestID >= 0) then
			local testVec2 = LM.Vector2:new_local()
			testVec2:Set(mesh:Point(closestID).fPos)

			-- convert the two chosen points to pixel coordinates
			-- this is to make sure that they are actually close enough in screen space
			local p1 = LM.Point:new_local()
			local p2 = LM.Point:new_local()
			local m = LM.Matrix:new_local()
			moho.layer:GetFullTransform(moho.frame, m, moho.document)
			m:Transform(testVec1)
			m:Transform(testVec2)
			mouseEvent.view:Graphics():WorldToScreen(testVec1, p1)
			mouseEvent.view:Graphics():WorldToScreen(testVec2, p2)
			p1.x = p1.x - p2.x
			p1.y = p1.y - p2.y
			local dist = (p1.x * p1.x) + (p1.y * p1.y)
			if (dist >= 100) then -- too far, don't auto-weld
				closestID = -1
			end
		end
	end

	if ((closestID < 0) and not(mouseEvent.altKey) and LM_TranslatePoints.autoWeld) then
		-- find a curve segment to auto-weld the freehand curve to
		curveID, segID = mouseEvent.view:PickEdge(mouseEvent.pt, curveID, segID)
	end

	self.bindStartTo = -1
	self.disableTaperStart = false
	self.disableTaperEnd = false
	if (self.autoClose) then -- no auto-welding on closed curves
		mesh:AddLonePoint(mouseEvent.vec, moho.frame)
		mesh:AppendPoint(mouseEvent.vec, moho.frame)
		mouseEvent.view:DrawMe()
	else
		if (closestID >= 0) then -- attach to an existing point
			self.disableTaperStart = true
			self.bindStartTo = closestID
			mesh:AddLonePoint(mouseEvent.vec, moho.frame)
			mesh:AppendPoint(mouseEvent.vec, moho.frame)
			mouseEvent.view:DrawMe()
		elseif (curveID >= 0) then -- add a point in the middle of some curve
			self.disableTaperStart = true
			mesh:AddPoint(mouseEvent.vec, curveID, segID, moho.frame)
			mouseEvent.view:DrawMe()
			mesh:AddPoint(mouseEvent.vec, -1, moho.frame)
			mesh:WeldPoints(mesh:CountPoints() - 2, mesh:CountPoints() - 3, moho.frame)
		else -- no auto-welding, just start a new curve
			mesh:AddLonePoint(mouseEvent.vec, moho.frame)
			mesh:AppendPoint(mouseEvent.vec, moho.frame)
			mouseEvent.view:DrawMe()
		end
	end

	self.previewWidth = moho:PixelToDoc(1)
	if (self.widthMode == 1 and mouseEvent.penPressure > 0) then -- pen pressure line width
		local ptWidth = LM.Lerp(mouseEvent.penPressure, self.minWidth, self.maxWidth)
		ptWidth = moho:PixelToDoc(ptWidth)
		mesh:Point(mesh:CountPoints() - 1).fWidth:SetValue(moho.frame, ptWidth)
		mesh:Point(mesh:CountPoints() - 2).fWidth:SetValue(moho.frame, ptWidth)
		self.previewWidth = ptWidth
	elseif (self.widthMode == 2) then -- random line width
		local ptWidth = LM.Lerp(math.random(), self.minWidth, self.maxWidth)
		ptWidth = moho:PixelToDoc(ptWidth)
		mesh:Point(mesh:CountPoints() - 1).fWidth:SetValue(moho.frame, ptWidth)
		mesh:Point(mesh:CountPoints() - 2).fWidth:SetValue(moho.frame, ptWidth)
		self.previewWidth = ptWidth
	end
	self.previousX = mouseEvent.startPt.x
	self.previousY = mouseEvent.startPt.y
	self.previousDrawX = mouseEvent.startPt.x
	self.previousDrawY = mouseEvent.startPt.y
	self.ptWidth = -1
end

function LM_Freehand:OnMouseMoved(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	-- move the most recent point to the current position, and set its width
	local n = mesh:CountPoints()
	local pt = mesh:Point(n - 1)
	pt.fPos:Set(mouseEvent.vec)
	pt.fAnimPos:SetValue(moho.frame, mouseEvent.vec)
	local ptWidth = -1
	if (self.widthMode == 1 and mouseEvent.penPressure > 0) then -- pen pressure line width
		ptWidth = LM.Lerp(mouseEvent.penPressure, self.minWidth, self.maxWidth)
		ptWidth = moho:PixelToDoc(ptWidth)
		pt.fWidth:SetValue(moho.frame, ptWidth)
	elseif (self.widthMode == 2) then -- random line width
		ptWidth = LM.Lerp(math.random(), self.minWidth, self.maxWidth)
		ptWidth = moho:PixelToDoc(ptWidth)
		pt.fWidth:SetValue(moho.frame, ptWidth)
	end

	-- test if the mouse has moved far enough for a new point
	testX = mouseEvent.pt.x - self.previousX
	testY = mouseEvent.pt.y - self.previousY
	local d = (testX * testX) + (testY * testY)
	if (d > self.pixelTolerance * self.pixelTolerance) then -- add a new point
		mesh:AppendPoint(mouseEvent.vec, moho.frame)
		pt = mesh:Point(mesh:CountPoints() - 1)
		pt.fWidth:SetValue(moho.frame, ptWidth)
		self.previousX = mouseEvent.pt.x
		self.previousY = mouseEvent.pt.y
		self.previewWidth = ptWidth
	end

	local g = mouseEvent.view:Graphics()
	local penWidth = moho:DocToPixel(self.previewWidth) * g:CurrentScale(false) * (g:Height() / moho.document:Height()) * 0.85

	g:SetSmoothing(true)
	g:Push()
	local m = g:CurrentTransform()
	m:Invert()
	g:ApplyMatrix(m)
	g:SetColor(255, 0, 0)
	g:SetPenWidth(penWidth)
	g:MoveTo(self.previousDrawX, self.previousDrawY)
	g:LineTo(mouseEvent.pt.x, mouseEvent.pt.y)
	g:Pop()
	g:SetSmoothing(false)
	mouseEvent.view:Refresh()
	self.previousDrawX = mouseEvent.pt.x
	self.previousDrawY = mouseEvent.pt.y
end

function LM_Freehand:OnMouseUp(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		mouseEvent.view:DrawMe()
		return
	end

	if (self.bindStartTo >= 0) then
		-- the initial point should be welded to an existing point
		-- we do this after drawing the curve to keep the two curves separate until done drawing
		mesh:WeldPoints(self.startingPoint, self.bindStartTo, moho.frame)
	end

	local n = mesh:CountPoints()
	if (mesh:CountPoints() > self.startingPoint + 2) then
		-- the last two points may lie on top of each other - if so, delete one
		local v = mesh:Point(n - 1).fPos - mesh:Point(n - 2).fPos
		if (v:Mag() < 1e-6) then
			mesh:Point(n - 2).fSelected = true
			mesh:DeletePoint(n - 1)
		end
	end

	if (mesh:CountPoints() <= self.startingPoint) then
		mesh:SelectNone()
		mouseEvent.view:DrawMe()
	end

	mesh:SelectNone()

	if (not(self.autoClose) and not(mouseEvent.altKey) and LM_TranslatePoints.autoWeld) then
		-- auto-weld the final point of the freehand curve
		if (self:WeldEndpoint(moho, mouseEvent.view, self.startingPoint, mesh:CountPoints() - 1)) then
			-- welded to an existing point
			self.disableTaperEnd = true
		else
			-- try welding the endpoint to a curve segment other than this freehand curve
			local ptID = mesh:CountPoints() - 1
			local m = LM.Matrix:new_local()
			local v = LM.Vector2:new_local()
			local pt = LM.Point:new_local()
			local curveID = -1
			local segID = -1

			moho.layer:GetFullTransform(moho.frame, m, moho.document)
			v:Set(mesh:Point(ptID).fPos)
			m:Transform(v)
			mouseEvent.view:Graphics():WorldToScreen(v, pt)
			curveID, segID = mesh:Point(ptID):GetEndpointEdge(curveID, segID)
			curveID, segID = mouseEvent.view:PickEdge(pt, curveID, -1)
			if (curveID >= 0) then -- add a point in the middle of some curve
				if (not mesh:Curve(curveID):IsPointOnSegment(ptID, segID)) then -- don't weld the point back on itself
					mesh:AddPoint(mesh:Point(ptID).fPos, curveID, segID, moho.frame)
					mesh:WeldPoints(ptID, mesh:CountPoints() - 1, moho.frame)
					self.disableTaperEnd = true
				end
			end
		end
	end

	if (self.taperStart or self.taperEnd) then
		-- taper the beginning and/or the end of the new line
		local f = moho:NewShapeLineWidth()
		local dist = 0
		local prevV = LM.Vector2:new_local()
		prevV:Set(mesh:Point(self.startingPoint).fPos)
		for i = self.startingPoint + 1, mesh:CountPoints() - 1 do
			local pt = mesh:Point(i)
			if (pt.fWidth.value < 0) then
				pt.fWidth:SetValue(moho.frame, f)
			end
			dist = dist + (pt.fPos - prevV):SquaredMag()
			prevV:Set(pt.fPos)
		end
		dist = dist / 4

		if (self.taperStart and not(self.disableTaperStart)) then
			-- beginning of the curve
			local n = 1
			local testDist = 0
			prevV:Set(mesh:Point(self.startingPoint).fPos)
			for i = self.startingPoint + 1, mesh:CountPoints() - 1 do
				local pt = mesh:Point(i)
				testDist = testDist + (pt.fPos - prevV):SquaredMag()
				prevV:Set(pt.fPos)
				if (testDist >= dist) then
					break
				end
				n = n + 1
			end
			for i = self.startingPoint, self.startingPoint + n - 1 do
				f = (i - self.startingPoint) / n
				local pt = mesh:Point(i)
				pt.fWidth:SetValue(moho.frame, pt.fWidth.value * f)
			end
		end

		if (self.taperEnd and not(self.disableTaperEnd)) then
			-- end of the curve
			local numP = mesh:CountPoints()
			local n = 1
			local testDist = 0
			prevV:Set(mesh:Point(numP - 1).fPos)
			for i = numP - 2, self.startingPoint, -1 do
				local pt = mesh:Point(i)
				testDist = testDist + (pt.fPos - prevV):SquaredMag()
				prevV:Set(pt.fPos)
				if (testDist >= dist) then
					break
				end
				n = n + 1
			end
			for i = numP - 1, numP - n, -1 do
				f = (numP - 1 - i) / n
				local pt = mesh:Point(i)
				pt.fWidth:SetValue(moho.frame, pt.fWidth.value * f)
			end
		end
	end

	if (self.angleTolerance > 0 and (mesh:CountPoints() > self.startingPoint + 2)) then
		-- adaptive freehand, remove "straight" sections of the curve
		local adaptiveLimit = math.cos(math.rad(self.angleTolerance))
		local runningVec = mesh:Point(self.startingPoint + 1).fPos - mesh:Point(self.startingPoint).fPos
		runningVec:NormMe()
		local i = self.startingPoint + 2
		while (i < mesh:CountPoints()) do
			local testVec = mesh:Point(i).fPos - mesh:Point(i - 1).fPos
			testVec:NormMe()
			if (runningVec:Dot(testVec) > adaptiveLimit) then
				mesh:DeletePoint(i - 1)
				i = i - 1
			else
				runningVec = testVec
			end
			i = i + 1
		end
	end

	if (self.autoClose) then
		-- close the shape
		local farOutVec = LM.Vector2:new_local()
		farOutVec:Set(1e6, 1e6)

		if (not (self.taperStart and self.taperEnd)) then
			-- equalize the widths at the endpoints, so they can be connected smoothly
			local tempW = mesh:Point(self.startingPoint + 1).fWidth.value
			mesh:Point(self.startingPoint).fWidth:SetValue(moho.frame, tempW)
			tempW = mesh:Point(mesh:CountPoints() - 2).fWidth.value
			mesh:Point(mesh:CountPoints() - 1).fWidth:SetValue(moho.frame, tempW)
		end

		mesh:AddPoint(farOutVec, -1, moho.frame)
		tempW = mesh:Point(mesh:CountPoints() - 3).fWidth.value
		mesh:Point(mesh:CountPoints() - 2).fWidth:SetValue(moho.frame, tempW)
		mesh:WeldPoints(mesh:CountPoints() - 2, mesh:CountPoints() - 3, moho.frame)
		tempW = mesh:Point(self.startingPoint).fWidth.value
		mesh:Point(mesh:CountPoints() - 1).fWidth:SetValue(moho.frame, tempW)
		mesh:WeldPoints(mesh:CountPoints() - 1, self.startingPoint, moho.frame)
	end

	-- select the points in this new curve
	for i = self.startingPoint, mesh:CountPoints() - 1 do
		mesh:Point(i).fSelected = true
	end
	if (self.bindStartTo >= 0) then
		mesh:Point(self.bindStartTo).fSelected = true
	end

	-- create a shape based on this new curve
	if (self.autoFill) then
		local shapeID = moho:CreateShape(true)
		if (shapeID >= 0) then
			if (not self.autoOutline) then
				mesh:Shape(shapeID).fHasOutline = false
			end
		end
	elseif (self.autoOutline) then
		moho:CreateShape(false)
	end
	moho:NewKeyframe(CHANNEL_FILL)
	moho:NewKeyframe(CHANNEL_LINE)

	moho:AddPointKeyframe(moho.frame)
	mouseEvent.view:DrawMe()
	moho:NewKeyframe(CHANNEL_POINT)
	moho:NewKeyframe(CHANNEL_CURVE)
	moho:UpdateSelectedChannels()
end

function LM_Freehand:OnKeyDown(moho, keyEvent)
	LM_SelectPoints:OnKeyDown(moho, keyEvent)
end

function LM_Freehand:WeldEndpoint(moho, view, startID, endID)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		mouseEvent.view:DrawMe()
		return false
	end

	local testVec1 = LM.Vector2:new_local()
	local testVec2 = LM.Vector2:new_local()

	testVec1:Set(mesh:Point(endID).fPos)
	local closestID = mesh:ClosestPoint(testVec1, endID, startID)

	if (closestID < 0) then
		return false
	end

	testVec2:Set(mesh:Point(closestID).fPos)

	-- convert the two chosen points to pixel coordinates
	-- this is to make sure that they are actually close enough in screen space
	local p1 = LM.Point:new_local()
	local p2 = LM.Point:new_local()
	local m = LM.Matrix:new_local()
	moho.layer:GetFullTransform(moho.frame, m, moho.document)
	m:Transform(testVec1)
	m:Transform(testVec2)
	view:Graphics():WorldToScreen(testVec1, p1)
	view:Graphics():WorldToScreen(testVec2, p2)
	p1.x = p1.x - p2.x
	p1.y = p1.y - p2.y
	closest = (p1.x * p1.x) + (p1.y * p1.y)
	if (closest < 100) then -- found one close enough to weld
		if (mesh:WeldPoints(endID, closestID, moho.frame)) then
			mesh:Point(closestID).fSelected = true
			return true
		end
	end

	return false
end

-- **************************************************
-- Freehand dialog
-- **************************************************

local LM_FreehandDialog = {}

function LM_FreehandDialog:new()
	local d = LM.GUI.SimpleDialog("Freehand Options", LM_FreehandDialog)
	local l = d:GetLayout()

	l:PushH()
		l:PushV()
			l:AddChild(LM.GUI.StaticText("Pixel tolerance"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Angle tolerance"), LM.GUI.ALIGN_LEFT)
		l:Pop()
		l:PushV()
			d.pixelText = LM.GUI.TextControl(0, "000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.pixelText, LM.GUI.ALIGN_LEFT)
			d.angleText = LM.GUI.TextControl(0, "000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.angleText, LM.GUI.ALIGN_LEFT)
		l:Pop()
	l:Pop()

	l:AddChild(LM.GUI.StaticText("Variable line width:"), LM.GUI.ALIGN_LEFT)
	l:Indent()

	d.lineWidthNone = LM.GUI.RadioButton("None", LM_Freehand.PRESSURE)
	l:AddChild(d.lineWidthNone, LM.GUI.ALIGN_LEFT)
	d.lineWidthPressure = LM.GUI.RadioButton("Use pen pressure", LM_Freehand.PRESSURE)
	l:AddChild(d.lineWidthPressure, LM.GUI.ALIGN_LEFT)
	d.lineWidthRandom = LM.GUI.RadioButton("Random", LM_Freehand.PRESSURE)
	l:AddChild(d.lineWidthRandom, LM.GUI.ALIGN_LEFT)

	l:PushH()
		l:Unindent()
		l:PushV()
			l:AddChild(LM.GUI.StaticText("Min width"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Max width"), LM.GUI.ALIGN_LEFT)
		l:Pop()
		l:PushV()
			d.minText = LM.GUI.TextControl(0, "00.000", 0, LM.GUI.FIELD_UFLOAT)
			d.minText:SetWheelInc(1)
			l:AddChild(d.minText, LM.GUI.ALIGN_LEFT)
			d.maxText = LM.GUI.TextControl(0, "00.000", 0, LM.GUI.FIELD_UFLOAT)
			d.maxText:SetWheelInc(1)
			l:AddChild(d.maxText, LM.GUI.ALIGN_LEFT)
		l:Pop()
	l:Pop()

	d.taperStartCheck = LM.GUI.CheckBox("Taper start")
	l:AddChild(d.taperStartCheck, LM.GUI.ALIGN_LEFT)

	d.taperEndCheck = LM.GUI.CheckBox("Taper end")
	l:AddChild(d.taperEndCheck, LM.GUI.ALIGN_LEFT)

	l:AddChild(LM.GUI.Button("Reset", LM_Freehand.RESET))

	return d
end

function LM_FreehandDialog:UpdateWidgets()
	self.pixelText:SetValue(LM_Freehand.pixelTolerance)
	self.angleText:SetValue(LM_Freehand.angleTolerance)
	if (LM_Freehand.widthMode == 1) then
		self.lineWidthPressure:SetValue(true)
	elseif (LM_Freehand.widthMode == 2) then
		self.lineWidthRandom:SetValue(true)
	else
		self.lineWidthNone:SetValue(true)
	end
	self.minText:SetValue(LM_Freehand.minWidth)
	self.maxText:SetValue(LM_Freehand.maxWidth)
	self.minText:Enable(LM_Freehand.widthMode > 0)
	self.maxText:Enable(LM_Freehand.widthMode > 0)
	self.taperStartCheck:SetValue(LM_Freehand.taperStart)
	self.taperEndCheck:SetValue(LM_Freehand.taperEnd)
end

function LM_FreehandDialog:OnValidate()
	local b = true
	if (not self:Validate(self.pixelText, 2, 1000)) then
		b = false
	end
	if (not self:Validate(self.angleText, 0, 30)) then
		b = false
	end
	if (not self:Validate(self.minText, 0, 32)) then
		b = false
	end
	if (not self:Validate(self.maxText, self.minText:FloatValue(), 32)) then
		b = false
	end
	return b
end

function LM_FreehandDialog:OnOK()
	LM_Freehand.pixelTolerance = self.pixelText:IntValue()
	LM_Freehand.angleTolerance = self.angleText:IntValue()
	if (self.lineWidthPressure:Value()) then
		LM_Freehand.widthMode = 1
	elseif (self.lineWidthRandom:Value()) then
		LM_Freehand.widthMode = 2
	else
		LM_Freehand.widthMode = 0
	end
	LM_Freehand.minWidth = self.minText:FloatValue()
	LM_Freehand.maxWidth = self.maxText:FloatValue()
	LM_Freehand.taperStart = self.taperStartCheck:Value()
	LM_Freehand.taperEnd = self.taperEndCheck:Value()
end

function LM_FreehandDialog:HandleMessage(msg)
	if (msg == LM_Freehand.PRESSURE) then
		self.minText:Enable(not self.lineWidthNone:Value())
		self.maxText:Enable(not self.lineWidthNone:Value())
	elseif (msg == LM_Freehand.RESET) then
		LM_Freehand.pixelTolerance = 10
		LM_Freehand.angleTolerance = 10
		LM_Freehand.widthMode = 1
		LM_Freehand.minWidth = 1
		LM_Freehand.maxWidth = 8
		LM_Freehand.taperStart = false
		LM_Freehand.taperEnd = false
		self:UpdateWidgets()
	end
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_Freehand.AUTOCLOSE = MOHO.MSG_BASE
LM_Freehand.AUTOFILL = MOHO.MSG_BASE + 1
LM_Freehand.AUTOOUTLINE = MOHO.MSG_BASE + 2
LM_Freehand.AUTOWELD = MOHO.MSG_BASE + 3
LM_Freehand.PRESSURE = MOHO.MSG_BASE + 4
LM_Freehand.RESET = MOHO.MSG_BASE + 5
LM_Freehand.pixelTolerance = 4
LM_Freehand.angleTolerance = 10
LM_Freehand.autoClose = false
LM_Freehand.autoFill = true
LM_Freehand.autoOutline = true
LM_Freehand.widthMode = 1 -- 0: none, 1: pen pressure, 2: random
LM_Freehand.minWidth = 1
LM_Freehand.maxWidth = 8
LM_Freehand.taperStart = false
LM_Freehand.taperEnd = false

function LM_Freehand:DoLayout(moho, layout)
	self.autoCloseCheck = LM.GUI.CheckBox("Auto close", self.AUTOCLOSE)
	layout:AddChild(self.autoCloseCheck)

	self.autoFillCheck = LM.GUI.CheckBox("Auto fill", self.AUTOFILL)
	layout:AddChild(self.autoFillCheck)

	self.autoOutlineCheck = LM.GUI.CheckBox("Auto outline", self.AUTOOUTLINE)
	layout:AddChild(self.autoOutlineCheck)

	self.dlog = LM_FreehandDialog:new()
	self.popup = LM.GUI.PopupDialog("Freehand Options")
	self.popup:SetDialog(self.dlog)
	layout:AddChild(self.popup)

	self.autoWeldCheck = LM.GUI.CheckBox("Auto-weld", self.AUTOWELD)
	layout:AddChild(self.autoWeldCheck)
end

function LM_Freehand:UpdateWidgets(moho)
	self.autoCloseCheck:SetValue(LM_Freehand.autoClose)
	self.autoFillCheck:SetValue(LM_Freehand.autoFill)
	self.autoOutlineCheck:SetValue(LM_Freehand.autoOutline)
	self.autoWeldCheck:SetValue(LM_TranslatePoints.autoWeld)
end

function LM_Freehand:HandleMessage(moho, view, msg)
	if (msg == self.AUTOCLOSE) then
		self.autoClose = self.autoCloseCheck:Value()
		if (self.autoFill and not(self.autoClose)) then
			self.autoFill = false
			self.autoFillCheck:SetValue(self.autoFill)
		end
	elseif (msg == self.AUTOFILL) then
		self.autoFill = self.autoFillCheck:Value()
		if (self.autoFill and not(self.autoClose)) then
			self.autoClose = true
			self.autoCloseCheck:SetValue(self.autoClose)
		end
	elseif (msg == self.AUTOOUTLINE) then
		self.autoOutline = self.autoOutlineCheck:Value()
	elseif (msg == self.AUTOWELD) then
		LM_TranslatePoints.autoWeld = self.autoWeldCheck:Value()
	end
end
