-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_LineWidth"

-- **************************************************
-- General information about this script
-- **************************************************

LM_LineWidth = {}

function LM_LineWidth:Name()
	return "Line Width"
end

function LM_LineWidth:Version()
	return "5.0"
end

function LM_LineWidth:Description()
	return "Drag side to side to adjust the line width through the selected points"
end

function LM_LineWidth:Creator()
	return "Lost Marble"
end

function LM_LineWidth:UILabel()
	return("Line Width")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_LineWidth.numSel = 0
LM_LineWidth.selID = -1

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_LineWidth:IsEnabled(moho)
	if (moho:CountPoints() > 0) then
		return true
	end
	return false
end

function LM_LineWidth:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	self.selID = -1
	self.selList = MOHO.SelectedPointList(mesh)
	self.numSel = table.getn(self.selList)

	if (self.numSel < 2) then -- work with only the closest selection
		-- find the closest point here
		self.selID = mesh:ClosestPoint(mouseEvent.startVec)
		if (self.selID >= 0) then
			self.numSel = 1
			mesh:SelectNone()
			local pt = mesh:Point(self.selID)
			pt.fSelected = true
			if (pt.fWidth.value < 0) then
				pt.fWidth:SetValue(moho.frame, 0)
			end
			pt.fTempWidth = pt.fWidth.value
			moho:UpdateSelectedChannels()
		end
		self.selList = MOHO.SelectedPointList(mesh)
		self:UpdateWidgets(moho)
	end

	for i, pt in self.selList do
		if (pt.fWidth.value < 0) then
			pt.fWidth:SetValue(moho.frame, 0)
		end
		pt.fTempWidth = pt.fWidth.value
	end

	mouseEvent.view:DrawMe()
end

function LM_LineWidth:OnMouseMoved(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local offset = moho:PixelToDoc((mouseEvent.pt.x - mouseEvent.startPt.x) / 16)
	local maxWidth = moho:PixelToDoc(64)

	if (self.numSel == 1) then
		local pt = mesh:Point(self.selID)
		pt.fWidth.value = pt.fTempWidth + offset
		pt.fWidth.value = LM.Clamp(pt.fWidth.value, 0, maxWidth)
		pt.fWidth:StoreValue()
	else
		for i, pt in self.selList do
			pt.fWidth.value = pt.fTempWidth + offset
			pt.fWidth.value = LM.Clamp(pt.fWidth.value, 0, maxWidth)
			pt.fWidth:StoreValue()
		end
	end

	mouseEvent.view:DrawMe()
	self:UpdateWidgets(moho)
end

function LM_LineWidth:OnMouseUp(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (self.numSel == 1) then
		-- update the selected list
		self.selList = MOHO.SelectedPointList(mesh)
	end
	-- clear out NULL width key values if there is more than one keyframe
	for i, pt in self.selList do
		if (pt.fWidth:CountKeys() > 1) then
			for id = pt.fWidth:CountKeys() - 1, 0, -1 do
				local when = pt.fWidth:GetKeyWhen(id)
				if (pt.fWidth:GetValue(when) < 0) then
					pt.fWidth:DeleteKey(when)
				end
			end
			if (pt.fWidth:CountKeys() < 1) then
				pt.fWidth:SetValue(0, -1)
			end
		end
	end

	self.selList = nil
	moho.layer:UpdateCurFrame()
	moho:NewKeyframe(CHANNEL_WIDTH)
	mouseEvent.view:DrawMe()
	moho:UpdateSelectedChannels()
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_LineWidth.DUMMY = MOHO.MSG_BASE
LM_LineWidth.CHANGE = MOHO.MSG_BASE + 1
LM_LineWidth.SELECTITEM = MOHO.MSG_BASE + 2

function LM_LineWidth:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu("Select Group")

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	layout:AddChild(LM.GUI.StaticText("Width:"))
	self.textW = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_UFLOAT)
	self.textW:SetWheelInc(1)
	layout:AddChild(self.textW)
end

function LM_LineWidth:UpdateWidgets(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	MOHO.BuildGroupMenu(self.menu, mesh, self.SELECTITEM, self.DUMMY)

	local width = 0
	local deleteSelList = false
	if (self.selList == nil) then
		self.selList = MOHO.SelectedPointList(mesh)
		deleteSelList = true
	end
	for i, pt in self.selList do
		width = width + pt.fWidth.value
	end
	if (table.getn(self.selList) < 1) then
		self.textW:SetValue("")
	else
		width = width / table.getn(self.selList)
		if (width < 0) then
			self.textW:SetValue("")
		else
			self.textW:SetValue(moho:DocToPixel(width))
		end
	end
	if (deleteSelList) then
		self.selList = nil
	end
end

function LM_LineWidth:HandleMessage(moho, view, msg)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (msg == self.CHANGE) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		local width = moho:PixelToDoc(self.textW:FloatValue())
		local selList = MOHO.SelectedPointList(mesh)
		for i, pt in selList do
			pt.fWidth:SetValue(moho.frame, width)
		end
	elseif (msg >= self.SELECTITEM) then
		mesh:SelectNone()
		local i = msg - self.SELECTITEM
		local name = mesh:Group(i):Name()
		mesh:SelectGroup(name)
		moho:UpdateUI()
	end
end
