-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_ManipulateBones"

-- **************************************************
-- General information about this script
-- **************************************************

LM_ManipulateBones = {}

function LM_ManipulateBones:Name()
	return "Manipulate Bones"
end

function LM_ManipulateBones:Version()
	return "5.0"
end

function LM_ManipulateBones:Description()
	return "Manipulate a group of bones (using inverse kinematics)"
end

function LM_ManipulateBones:Creator()
	return "Lost Marble"
end

function LM_ManipulateBones:UILabel()
	return("Manipulate Bones")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_ManipulateBones.selID = -1
LM_ManipulateBones.lastVec = LM.Vector2:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_ManipulateBones:UseParentLayer(moho)
	if (moho.layer:IsBoneType()) then
		return false
	end
	return true
end

function LM_ManipulateBones:Skeleton(moho)
	local skel = moho:Skeleton()
	if ((skel == nil) and (moho.frame == 0)) then
		skel = moho:ParentSkeleton() -- at frame 0, this tool acts as a bone tester, and can be used on the parent layer's skeleton
	end
	return skel
end

function LM_ManipulateBones:IsEnabled(moho)
	local skel = self:Skeleton(moho)
	if (skel == nil) then
		return false
	end
	if (skel:CountBones() < 1) then
		return false
	end
	return true
end

function LM_ManipulateBones:OnMouseDown(moho, mouseEvent)
	local skel = self:Skeleton(moho)
	if (skel == nil) then
		return
	end

	local testMode = (moho.frame == 0)

	if (self:UseParentLayer(moho)) then
		-- bone manipulation must happen in the bone layer's coordinate system to make sense
		local parentM = LM.Matrix:new_local()
		moho.layer:Parent():GetFullTransform(moho.frame, parentM, moho.document)
		mouseEvent.vec:Set(mouseEvent.view:Point2Vec(mouseEvent.pt, parentM))
		mouseEvent.startVec:Set(mouseEvent.view:Point2Vec(mouseEvent.startPt, parentM))
	end

	if (not testMode) then
		moho.document:SetDirty()
	end
	if (self:UseParentLayer(moho)) then
		moho.document:PrepUndo(moho.layer:Parent())
		self.selID = mouseEvent.view:PickBone(mouseEvent.pt, mouseEvent.vec, moho.layer:Parent(), false)
	else
		moho.document:PrepUndo(moho.layer)
		self.selID = mouseEvent.view:PickBone(mouseEvent.pt, mouseEvent.vec, moho.layer, false)
	end

	for i = 0, skel:CountBones() - 1 do
		local bone = skel:Bone(i)

		bone.fTempPos = bone.fPos
		bone.fTempAngle = bone.fAngle
		if (i == self.selID) then
			bone.fSelected = true
			if (not testMode) then
				if ((bone.fParent < 0) or (self:CountBoneChildren(skel, bone.fParent, true) > 1)) then
					-- case 1: rotate an individual bone
						-- happens when:
						-- bone has no parent, or
						-- bone is one of multiple children
					bone.fAnimAngle:SetValue(moho.frame, bone.fAngle)
				else
					-- case 2: interesting case - part of an only-child chain - use IK solver
					bone.fAnimAngle:SetValue(moho.frame, bone.fAngle)
					while bone.fParent >= 0 do
						if (self:CountBoneChildren(skel, bone.fParent, true) > 1) then
							break
						end
						bone = skel:Bone(bone.fParent)
						bone.fAnimAngle:SetValue(moho.frame, bone.fAngle)
					end
				end
			end
		else
			bone.fSelected = false
		end
	end

	moho:UpdateBonePointSelection()
	mouseEvent.view:DrawMe()
	self.lastVec:Set(mouseEvent.vec)
end

function LM_ManipulateBones:OnMouseMoved(moho, mouseEvent)
	local skel = self:Skeleton(moho)
	if ((skel == nil) or (self.selID < 0)) then
		return
	end

	local testMode = (moho.frame == 0)

	if (self:UseParentLayer(moho)) then
		-- bone manipulation must happen in the bone layer's coordinate system to make sense
		local parentM = LM.Matrix:new_local()
		moho.layer:Parent():GetFullTransform(moho.frame, parentM, moho.document)
		mouseEvent.vec:Set(mouseEvent.view:Point2Vec(mouseEvent.pt, parentM))
		mouseEvent.startVec:Set(mouseEvent.view:Point2Vec(mouseEvent.startPt, parentM))
	end

	local bone = skel:Bone(self.selID)

	if ((bone.fParent < 0) or (self:CountBoneChildren(skel, bone.fParent, true) > 1)) then
		-- case 1: rotate an individual bone
			-- happens when:
			-- bone has no parent, or
			-- bone is one of multiple children
		bone.fPos = bone.fTempPos
		bone.fAngle = bone.fTempAngle
		skel:UpdateBoneMatrix(self.selID)

		local baseVec = LM.Vector2:new_local()

		baseVec:Set(0, 0)
		bone.fMovedMatrix:Transform(baseVec)
		local v1 = self.lastVec - baseVec
		local v2 = mouseEvent.vec - baseVec
		v2:Rotate(-math.atan2(v1.y, v1.x))
		bone.fAngle = bone.fAngle + math.atan2(v2.y, v2.x)

		if (bone.fConstraints) then
			local min = bone.fAnimAngle:GetValue(0)
			local max = min + bone.fMaxConstraint
			min = min + bone.fMinConstraint
			bone.fAngle = LM.Clamp(bone.fAngle, min, max)
		end
		bone.fTempAngle = bone.fAngle
		if (not testMode) then
			bone.fAnimAngle:SetValue(moho.frame, bone.fAngle)
		end
	else
		-- case 2: interesting case - part of an only-child chain - use IK solver
		skel:UpdateBoneMatrix(self.selID)

		local tipVec = LM.Vector2:new_local()

		tipVec:Set(bone.fLength, 0)
		bone.fMovedMatrix:Transform(tipVec)
		tipVec = tipVec + (mouseEvent.vec - self.lastVec)

		skel:IKAngleSolver(self.selID, tipVec)
		if (not testMode) then
			bone.fAnimAngle:SetValue(moho.frame, bone.fAngle)
			while bone.fParent >= 0 do
				if (self:CountBoneChildren(skel, bone.fParent, true) > 1) then
					break
				end
				bone = skel:Bone(bone.fParent)
				bone.fAnimAngle:SetValue(moho.frame, bone.fAngle)
			end
		end
	end

	skel:UpdateBoneMatrix()
	if (not testMode) then
		moho.layer:UpdateCurFrame()
	end
	mouseEvent.view:DrawMe()
	self.lastVec:Set(mouseEvent.vec)
end

function LM_ManipulateBones:OnMouseUp(moho, mouseEvent)
	local skel = self:Skeleton(moho)
	if ((skel == nil) or (self.selID < 0)) then
		return
	end

	local testMode = (moho.frame == 0)

	if ((self.selID >= 0) and (not testMode)) then
		moho.layer:UpdateCurFrame()
		moho:NewKeyframe(CHANNEL_BONE)
	end

	self.selID = -1

	mouseEvent.view:DrawMe()
end

function LM_ManipulateBones:CountBoneChildren(skel, boneID, ignoreControlledBones)
	local n = 0
	for i = 0, skel:CountBones() - 1 do
		local bone = skel:Bone(i)
		if (bone.fParent == boneID) then
			n = n + 1
			if (ignoreControlledBones) then
				if (bone.fAngleControlParent >= 0 or bone.fPosControlParent >= 0 or bone.fScaleControlParent >= 0 or bone.fBoneDynamics.value) then
					n = n - 1 -- ignore this bone, as it is not free to move by itself
				end
			end
		end
	end
	return n
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************
--[[
LM_ManipulateBones.AUTOFILL = MOHO.MSG_BASE
LM_ManipulateBones.AUTOOUTLINE = MOHO.MSG_BASE + 1
LM_ManipulateBones.autoFill = true
LM_ManipulateBones.autoOutline = true

function LM_ManipulateBones:DoLayout(moho, layout)
	self.autoFillCheck = LM.GUI.CheckBox("Auto fill", self.AUTOFILL)
	layout:AddChild(self.autoFillCheck)

	self.autoOutlineCheck = LM.GUI.CheckBox("Auto outline", self.AUTOOUTLINE)
	layout:AddChild(self.autoOutlineCheck)
end

function LM_ManipulateBones:UpdateWidgets(moho)
	self.autoFillCheck:SetValue(self.autoFill)
	self.autoOutlineCheck:SetValue(self.autoOutline)
end

function LM_ManipulateBones:HandleMessage(moho, view, msg)
	if (msg == self.AUTOFILL) then
		self.autoFill = self.autoFillCheck:Value()
	elseif (msg == self.AUTOOUTLINE) then
		self.autoOutline = self.autoOutlineCheck:Value()
	end
end
]]
