-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_Rectangle"

-- **************************************************
-- General information about this script
-- **************************************************

LM_Rectangle = {}

function LM_Rectangle:Name()
	return "Rectangle"
end

function LM_Rectangle:Version()
	return "5.0"
end

function LM_Rectangle:Description()
	return "Draw a rectangle (hold <shift> to constrain, <alt> to center)"
end

function LM_Rectangle:Creator()
	return "Lost Marble"
end

function LM_Rectangle:UILabel()
	return("Rectangle")
end

function LM_Rectangle:LoadPrefs(prefs)
	self.autoFill = prefs:GetBool("LM_Rectangle.autoFill", true)
	self.autoOutline = prefs:GetBool("LM_Rectangle.autoOutline", true)
end

function LM_Rectangle:SavePrefs(prefs)
	prefs:SetBool("LM_Rectangle.autoFill", self.autoFill)
	prefs:SetBool("LM_Rectangle.autoOutline", self.autoOutline)
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_Rectangle.startVec = LM.Vector2:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_Rectangle:IsEnabled(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return false
	end
	if (moho.layer:CurrentAction() ~= "") then
		return false -- creating new objects in the middle of an action can lead to unexpected results
	end
	return true
end

function LM_Rectangle:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	local n = mesh:CountPoints()

	mesh:SelectNone()

	self.startVec:Set(mouseEvent.vec)
	if (moho.gridOn) then
		moho:SnapToGrid(self.startVec)
	end

	mesh:AddLonePoint(self.startVec, moho.frame)
	mesh:AppendPoint(self.startVec, moho.frame)
	mesh:AppendPoint(self.startVec, moho.frame)
	mesh:AppendPoint(self.startVec, moho.frame)
	mesh:AppendPoint(self.startVec, moho.frame)

	mesh:WeldPoints(n + 4, n, 0)

	mesh:Point(n):SetCurvature(MOHO.PEAKED, moho.frame)
	mesh:Point(n + 1):SetCurvature(MOHO.PEAKED, moho.frame)
	mesh:Point(n + 2):SetCurvature(MOHO.PEAKED, moho.frame)
	mesh:Point(n + 3):SetCurvature(MOHO.PEAKED, moho.frame)

	mesh:SelectConnected()
	if (self.autoFill) then
		local shapeID = moho:CreateShape(true)
		if (shapeID >= 0) then
			if (not self.autoOutline) then
				mesh:Shape(shapeID).fHasOutline = false
			end
		end
	elseif (self.autoOutline) then
		moho:CreateShape(false)
	end

	mouseEvent.view:DrawMe()
end

function LM_Rectangle:OnMouseMoved(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local vec = LM.Vector2:new_local()
	vec:Set(mouseEvent.vec)
	if (moho.gridOn) then
		moho:SnapToGrid(vec)
	end

	local x1 = self.startVec.x
	local x2 = vec.x
	local y1 = self.startVec.y
	local y2 = vec.y

	if (mouseEvent.shiftKey) then
		-- constrain width = height
		if (x2 > x1) then
			if (y2 > y1) then
				y2 = y1 + (x2 - x1)
			else
				y2 = y1 - (x2 - x1)
			end
		else
			if (y2 > y1) then
				y2 = y1 - (x2 - x1)
			else
				y2 = y1 + (x2 - x1)
			end
		end
	end
	if (mouseEvent.altKey) then
		-- draw from center point
		x1 = x1 - (x2 - x1)
		y1 = y1 - (y2 - y1)
	end

	local n = mesh:CountPoints()

	local pt = mesh:Point(n - 4)
	pt.fPos.x = x1
	pt.fPos.y = y1
	local pt = mesh:Point(n - 3)
	pt.fPos.x = x2
	pt.fPos.y = y1
	local pt = mesh:Point(n - 2)
	pt.fPos.x = x2
	pt.fPos.y = y2
	local pt = mesh:Point(n - 1)
	pt.fPos.x = x1
	pt.fPos.y = y2

	moho:AddPointKeyframe(moho.frame)
	mouseEvent.view:DrawMe()
end

function LM_Rectangle:OnMouseUp(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho:AddPointKeyframe(moho.frame)
	mouseEvent.view:DrawMe()
	moho:NewKeyframe(CHANNEL_POINT)
	moho:NewKeyframe(CHANNEL_CURVE)
	moho:UpdateSelectedChannels()
end

function LM_Rectangle:OnKeyDown(moho, keyEvent)
	LM_SelectPoints:OnKeyDown(moho, keyEvent)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_Rectangle.AUTOFILL = MOHO.MSG_BASE
LM_Rectangle.AUTOOUTLINE = MOHO.MSG_BASE + 1
LM_Rectangle.autoFill = true
LM_Rectangle.autoOutline = true

function LM_Rectangle:DoLayout(moho, layout)
	self.autoFillCheck = LM.GUI.CheckBox("Auto fill", self.AUTOFILL)
	layout:AddChild(self.autoFillCheck)

	self.autoOutlineCheck = LM.GUI.CheckBox("Auto outline", self.AUTOOUTLINE)
	layout:AddChild(self.autoOutlineCheck)
end

function LM_Rectangle:UpdateWidgets(moho)
	self.autoFillCheck:SetValue(self.autoFill)
	self.autoOutlineCheck:SetValue(self.autoOutline)
end

function LM_Rectangle:HandleMessage(moho, view, msg)
	if (msg == self.AUTOFILL) then
		self.autoFill = self.autoFillCheck:Value()
	elseif (msg == self.AUTOOUTLINE) then
		self.autoOutline = self.autoOutlineCheck:Value()
	end
end
