-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_RotatePoints"

-- **************************************************
-- General information about this script
-- **************************************************

LM_RotatePoints = {}

function LM_RotatePoints:Name()
	return "Rotate Points"
end

function LM_RotatePoints:Version()
	return "5.0"
end

function LM_RotatePoints:Description()
	return "Rotate selected points (hold <shift> to constrain)"
end

function LM_RotatePoints:Creator()
	return "Lost Marble"
end

function LM_RotatePoints:UILabel()
	return("Rotate Points")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_RotatePoints.dragging = false
LM_RotatePoints.startAngle = 0
LM_RotatePoints.lastVec = LM.Vector2:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_RotatePoints:IsEnabled(moho)
	if (moho:CountSelectedPoints() > 1) then
		return true
	end
	return false
end

function LM_RotatePoints:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end
	self.dragging = true

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	mesh:PrepMovePoints()
	self.centerVec = mesh:SelectedCenter()
	self.startAngle = 0

	self.lastVec:Set(mouseEvent.vec)
	mouseEvent.view:DrawMe()
end

function LM_RotatePoints:OnMouseMoved(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local angle = self.startAngle
	local v1 = self.lastVec - self.centerVec
	local v2 = mouseEvent.vec - self.centerVec
	v2:Rotate(-math.atan2(v1.y, v1.x))
	angle = angle + math.atan2(v2.y, v2.x)
	self.startAngle = angle
	if (mouseEvent.shiftKey) then
		angle = angle / (math.pi / 4)
		angle = (math.pi / 4) * LM.Round(angle)
	end

	mesh:RotatePoints(angle, self.centerVec)
	moho:AddPointKeyframe(moho.frame)

	self.lastVec:Set(mouseEvent.vec)
	mouseEvent.view:DrawMe()
end

function LM_RotatePoints:OnMouseUp(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho:AddPointKeyframe(moho.frame)
	moho:NewKeyframe(CHANNEL_POINT)

	self.dragging = false

	mouseEvent.view:DrawMe()
end

function LM_RotatePoints:OnKeyDown(moho, keyEvent)
	LM_SelectPoints:OnKeyDown(moho, keyEvent)
end

function LM_RotatePoints:DrawMe(moho, view)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local g = view:Graphics()
	local centerVec = LM.Vector2:new_local()
	local matrix = LM.Matrix:new_local()

	moho.layer:GetFullTransform(moho.frame, matrix, moho.document)
	g:Push()
	g:ApplyMatrix(matrix)

	if (self.dragging) then
		centerVec:Set(self.centerVec)
	else
		centerVec = mesh:SelectedCenter()
	end
	g:SetColor(255, 0, 0)
	g:SetSmoothing(true)
	g:DrawLine(centerVec.x - 0.05, centerVec.y, centerVec.x + 0.05, centerVec.y)
	g:DrawLine(centerVec.x, centerVec.y - 0.05, centerVec.x, centerVec.y + 0.05)

	g:Pop()
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_RotatePoints.DUMMY = MOHO.MSG_BASE
LM_RotatePoints.MODIFY = MOHO.MSG_BASE + 1
LM_RotatePoints.RESET = MOHO.MSG_BASE + 2
LM_RotatePoints.SELECTITEM = MOHO.MSG_BASE + 3

function LM_RotatePoints:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu("Select Group")

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	layout:AddChild(LM.GUI.StaticText("Rotation:"))

	self.angle = LM.GUI.TextControl(0, "000.0000", 0, LM.GUI.FIELD_FLOAT)
	self.angle:SetWheelInc(5)
	layout:AddChild(self.angle)

	layout:AddChild(LM.GUI.Button("Modify", self.MODIFY))
	layout:AddChild(LM.GUI.Button("Reset", self.RESET))
end

function LM_RotatePoints:UpdateWidgets(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	MOHO.BuildGroupMenu(self.menu, mesh, self.SELECTITEM, self.DUMMY)

	self.angle:SetValue(0)
end

function LM_RotatePoints:HandleMessage(moho, view, msg)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (msg == self.MODIFY) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		local centerVec = mesh:SelectedCenter()
		local angle = math.rad(self.angle:FloatValue())

		mesh:PrepMovePoints()
		mesh:RotatePoints(angle, centerVec)
		moho:AddPointKeyframe(moho.frame)
		moho:NewKeyframe(CHANNEL_POINT)
	elseif (msg == self.RESET) then
		if (moho.frame > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()

			for i = 0, mesh:CountPoints() - 1 do
				local pt = mesh:Point(i)
				if (pt.fSelected) then
					pt:SetPos(pt.fAnimPos:GetValue(0), moho.frame)
				end
			end
		end
		moho:NewKeyframe(CHANNEL_POINT)
	elseif (msg >= self.SELECTITEM) then
		mesh:SelectNone()
		local i = msg - self.SELECTITEM
		local name = mesh:Group(i):Name()
		mesh:SelectGroup(name)
		moho:UpdateUI()
	end
end
