-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_ScaleLayer"

-- **************************************************
-- General information about this script
-- **************************************************

LM_ScaleLayer = {}

function LM_ScaleLayer:Name()
	return "Scale Layer"
end

function LM_ScaleLayer:Version()
	return "5.0"
end

function LM_ScaleLayer:Description()
	return "Resize entire layer (hold <alt> to preserve volume for horizontal or vertical scale)"
end

function LM_ScaleLayer:Creator()
	return "Lost Marble"
end

function LM_ScaleLayer:UILabel()
	return("Scale Layer")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_ScaleLayer.scalingDir = -1
LM_ScaleLayer.startVal = LM.Vector3:new_local()
LM_ScaleLayer.matrix = LM.Matrix:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_ScaleLayer:OnMouseDown(moho, mouseEvent)
	self.scalingDir = -1
	-- First step - figure out which scaling handle is being dragged
	local origin = moho.layer:Origin()
	local dx = math.abs(mouseEvent.vec.x - origin.x)
	local dy = math.abs(mouseEvent.vec.y - origin.y)

	if (dx > 0.4 and dx < 0.6) then
		if (dy > 0.4 and dy < 0.6) then
			self.scalingDir = 2
		elseif (dy < 0.1) then
			self.scalingDir = 0
		end
	elseif (dx < 0.1) then
		if (dy > 0.4 and dy < 0.6) then
			self.scalingDir = 1
		end
	end

	if (self.scalingDir < 0 and dx > 0.4 and dy > 0.4) then
		self.scalingDir = 2
	end

	if (self.scalingDir < 0) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	self.startVal:Set(moho.layer.fScale.value)
	moho.layer.fScale:AddKey(moho.frame)

	moho.layer:GetFullTransform(moho.frame, self.matrix, moho.document)

	mouseEvent.view:DrawMe()
end

function LM_ScaleLayer:OnMouseMoved(moho, mouseEvent)
	if (self.scalingDir < 0) then
		return
	end

	moho.layer.fScale:SetValue(moho.frame, self.startVal)

	local startVec = mouseEvent.view:Point2Vec(mouseEvent.startPt, self.matrix)
	local nextVec = mouseEvent.view:Point2Vec(mouseEvent.pt, self.matrix)

	-- scaling connected to actual drag amount
	local origin = moho.layer:Origin()
	local v1 = startVec - origin
	local v2 = nextVec - origin
	local scaling = LM.Vector3:new_local()
	scaling:Set(v2.x / v1.x, v2.y / v1.y, 1)

	if (self.scalingDir == 0) then
		scaling.y = 1
		if (mouseEvent.altKey) then
			scaling.y = 1 / scaling.x
		end
	elseif (self.scalingDir == 1) then
		scaling.x = 1
		if (mouseEvent.altKey) then
			scaling.x = 1 / scaling.y
		end
	else
		scaling.x = (scaling.x + scaling.y) / 2
		scaling.y = scaling.x
		scaling.z = scaling.x
	end

	local vec = LM.Vector3:new_local()
	vec:Set(self.startVal)
	vec.x = vec.x * scaling.x
	vec.y = vec.y * scaling.y
	vec.z = vec.z * scaling.z
	moho.layer.fScale:SetValue(moho.frame, vec)

	moho.document:DepthSort()
	mouseEvent.view:DrawMe()
end

function LM_ScaleLayer:OnMouseUp(moho, mouseEvent)
	if (self.scalingDir < 0) then
		return
	end

	moho:NewKeyframe(CHANNEL_LAYER_S)
end

function LM_ScaleLayer:DrawMe(moho, view)
	local g = view:Graphics()
	local origin = moho.layer:Origin()
	local markerR = 4
	local matrix = LM.Matrix:new_local()

	moho.layer:GetFullTransform(moho.frame, matrix, moho.document)
	g:Push()
	g:ApplyMatrix(matrix)

	g:SetColor(255, 0, 0)
	g:SetSmoothing(true)
	g:DrawLine(origin.x - 0.5, origin.y + 0.5, origin.x - 0.5, origin.y - 0.5)
	g:DrawLine(origin.x + 0.5, origin.y + 0.5, origin.x + 0.5, origin.y - 0.5)
	g:DrawLine(origin.x + 0.5, origin.y + 0.5, origin.x - 0.5, origin.y + 0.5)
	g:DrawLine(origin.x + 0.5, origin.y - 0.5, origin.x - 0.5, origin.y - 0.5)
	g:SetSmoothing(false)
	g:DrawFatMarker(origin.x - 0.5, origin.y - 0.5, markerR)
	g:DrawFatMarker(origin.x - 0.5, origin.y + 0.5, markerR)
	g:DrawFatMarker(origin.x + 0.5, origin.y - 0.5, markerR)
	g:DrawFatMarker(origin.x + 0.5, origin.y + 0.5, markerR)
	g:DrawFatMarker(origin.x - 0.5, origin.y, markerR)
	g:DrawFatMarker(origin.x + 0.5, origin.y, markerR)
	g:DrawFatMarker(origin.x, origin.y - 0.5, markerR)
	g:DrawFatMarker(origin.x, origin.y + 0.5, markerR)

	g:Pop()
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_ScaleLayer.CHANGE = MOHO.MSG_BASE
LM_ScaleLayer.RESET = MOHO.MSG_BASE + 1

function LM_ScaleLayer:DoLayout(moho, layout)
	layout:AddChild(LM.GUI.StaticText("Scale"))

	layout:AddChild(LM.GUI.StaticText("X:"))
	self.textX = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.textX)

	layout:AddChild(LM.GUI.StaticText("Y:"))
	self.textY = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.textY)

	layout:AddChild(LM.GUI.StaticText("Z:"))
	self.textZ = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.textZ)

	layout:AddChild(LM.GUI.Button("Reset", self.RESET))
end

function LM_ScaleLayer:UpdateWidgets(moho)
	self.textX:SetValue(moho.layer.fScale.value.x)
	self.textY:SetValue(moho.layer.fScale.value.y)
	self.textZ:SetValue(moho.layer.fScale.value.z)
end

function LM_ScaleLayer:HandleMessage(moho, view, msg)
	local newVal = LM.Vector3:new_local()

	if (msg == self.RESET) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		if (moho.frame == 0) then
			newVal:Set(1, 1, 1)
			moho.layer.fScale:SetValue(0, newVal)
		else
			newVal:Set(moho.layer.fScale:GetValue(0))
			moho.layer.fScale:SetValue(moho.frame, newVal)
		end
		self:UpdateWidgets(moho)
		moho:NewKeyframe(CHANNEL_LAYER_S)
	elseif (msg == self.CHANGE) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		newVal.x = self.textX:FloatValue()
		newVal.y = self.textY:FloatValue()
		newVal.z = self.textZ:FloatValue()
		local vec = moho.layer.fScale:GetValue(moho.frame) - newVal
		if (vec:Mag() > 0.0001) then
			moho.layer.fScale:SetValue(moho.frame, newVal)
			moho:NewKeyframe(CHANNEL_LAYER_S)
		end
	end
end
