-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_ScalePoints"

-- **************************************************
-- General information about this script
-- **************************************************

LM_ScalePoints = {}

function LM_ScalePoints:Name()
	return "Scale Points"
end

function LM_ScalePoints:Version()
	return "5.0"
end

function LM_ScalePoints:Description()
	return "Scale selected points (hold <alt> to preserve volume)"
end

function LM_ScalePoints:Creator()
	return "Lost Marble"
end

function LM_ScalePoints:UILabel()
	return("Scale Points")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_ScalePoints.scalingDir = -1
LM_ScalePoints.minVec = LM.Vector2:new_local()
LM_ScalePoints.maxVec = LM.Vector2:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_ScalePoints:IsEnabled(moho)
	if (moho:CountSelectedPoints() > 1) then
		return true
	end
	return false
end

function LM_ScalePoints:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	mesh:SelectedBounds(self.minVec, self.maxVec)
	self.centerVec = (self.minVec + self.maxVec) / 2

	-- First step - figure out which scaling handle is being dragged
	self.scalingDir = -1
	local boxW = (self.maxVec.x - self.minVec.x) / 2
	local boxH = (self.maxVec.y - self.minVec.y) / 2
	dx = mouseEvent.vec.x - self.centerVec.x
	dx = math.abs(dx)
	dy = mouseEvent.vec.y - self.centerVec.y
	dy = math.abs(dy)
	if ((dx > 0.8 * boxW) and (dx < 1.2 * boxW)) then
		if ((dy > 0.8 * boxH) and (dy < 1.2 * boxH)) then
			self.scalingDir = 2
		elseif (dy < 0.2 * boxH) then
			self.scalingDir = 0
		end
	elseif (dx < 0.2 * boxW) then
		if ((dy > 0.8 * boxH) and (dy < 1.2 * boxH)) then
			self.scalingDir = 1
		end
	end

	if ((self.scalingDir < 0) and (dx > 0.8 * boxW) and (dy > 0.8 * boxH)) then
		self.scalingDir = 2
	end

	if (self.scalingDir < 0) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	mesh:PrepMovePoints()

	mouseEvent.view:DrawMe()
end

function LM_ScalePoints:OnMouseMoved(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (self.scalingDir < 0) then
		return
	end

	local scaling = LM.Vector2:new_local()

	scaling:Set(1, 1)
	-- scaling connected to actual drag amount
	local v1 = mouseEvent.startVec - self.centerVec
	local v2 = mouseEvent.vec - self.centerVec
	scaling.x = v2.x / v1.x
	scaling.y = v2.y / v1.y

	if (self.scalingDir == 0) then
		scaling.y = 1
		if (mouseEvent.altKey) then
			scaling.y = 1 / scaling.x
		end
	elseif (self.scalingDir == 1) then
		scaling.x = 1
		if (mouseEvent.altKey) then
			scaling.x = 1 / scaling.y
		end
	else
		scaling.x = (scaling.x + scaling.y) / 2
		scaling.y = scaling.x
	end

	mesh:ScalePoints(scaling.x, scaling.y, self.centerVec)
	moho:AddPointKeyframe(moho.frame)

	mouseEvent.view:DrawMe()
end

function LM_ScalePoints:OnMouseUp(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (self.scalingDir < 0) then
		return
	end

	moho:AddPointKeyframe(moho.frame)
	moho:NewKeyframe(CHANNEL_POINT)

	mouseEvent.view:DrawMe()
end

function LM_ScalePoints:OnKeyDown(moho, keyEvent)
	LM_SelectPoints:OnKeyDown(moho, keyEvent)
end

function LM_ScalePoints:DrawMe(moho, view)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local g = view:Graphics()
	local min = LM.Vector2:new_local()
	local max = LM.Vector2:new_local()
	local markerR = 4
	local matrix = LM.Matrix:new_local()

	mesh:SelectedBounds(min, max)

	moho.layer:GetFullTransform(moho.frame, matrix, moho.document)
	g:Push()
	g:ApplyMatrix(matrix)

	g:SetColor(255, 0, 0)
	g:SetSmoothing(true)
	g:DrawLine(min.x, min.y, max.x, min.y)
	g:DrawLine(min.x, max.y, max.x, max.y)
	g:DrawLine(min.x, min.y, min.x, max.y)
	g:DrawLine(max.x, min.y, max.x, max.y)
	g:SetSmoothing(false)
	g:DrawFatMarker(min.x, min.y, markerR)
	g:DrawFatMarker(min.x, max.y, markerR)
	g:DrawFatMarker(max.x, min.y, markerR)
	g:DrawFatMarker(max.x, max.y, markerR)
	g:DrawFatMarker((min.x + max.x) / 2, min.y, markerR)
	g:DrawFatMarker((min.x + max.x) / 2, max.y, markerR)
	g:DrawFatMarker(min.x, (min.y + max.y) / 2, markerR)
	g:DrawFatMarker(max.x, (min.y + max.y) / 2, markerR)

	g:Pop()
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_ScalePoints.DUMMY = MOHO.MSG_BASE
LM_ScalePoints.MODIFY = MOHO.MSG_BASE + 1
LM_ScalePoints.RESET = MOHO.MSG_BASE + 2
LM_ScalePoints.SELECTITEM = MOHO.MSG_BASE + 3

function LM_ScalePoints:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu("Select Group")

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	layout:AddChild(LM.GUI.StaticText("Scale"))

	layout:AddChild(LM.GUI.StaticText("X:"))
	self.scaleX = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.scaleX)

	layout:AddChild(LM.GUI.StaticText("Y:"))
	self.scaleY = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.scaleY)

	layout:AddChild(LM.GUI.Button("Modify", self.MODIFY))
	layout:AddChild(LM.GUI.Button("Reset", self.RESET))
end

function LM_ScalePoints:UpdateWidgets(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	MOHO.BuildGroupMenu(self.menu, mesh, self.SELECTITEM, self.DUMMY)

	self.scaleX:SetValue(1)
	self.scaleY:SetValue(1)
end

function LM_ScalePoints:HandleMessage(moho, view, msg)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (msg == self.MODIFY) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		local centerVec = mesh:SelectedCenter()
		local scaleX = self.scaleX:FloatValue()
		local scaleY = self.scaleY:FloatValue()

		mesh:PrepMovePoints()
		mesh:ScalePoints(scaleX, scaleY, centerVec)
		moho:AddPointKeyframe(moho.frame)
		moho:NewKeyframe(CHANNEL_POINT)
	elseif (msg == self.RESET) then
		if (moho.frame > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()

			for i = 0, mesh:CountPoints() - 1 do
				local pt = mesh:Point(i)
				if (pt.fSelected) then
					pt:SetPos(pt.fAnimPos:GetValue(0), moho.frame)
				end
			end
		end
		moho:NewKeyframe(CHANNEL_POINT)
	elseif (msg >= self.SELECTITEM) then
		mesh:SelectNone()
		local i = msg - self.SELECTITEM
		local name = mesh:Group(i):Name()
		mesh:SelectGroup(name)
		moho:UpdateUI()
	end
end
