-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_SelectBone"

-- **************************************************
-- General information about this script
-- **************************************************

LM_SelectBone = {}

function LM_SelectBone:Name()
	return "Select Bone"
end

function LM_SelectBone:Version()
	return "5.0"
end

function LM_SelectBone:Description()
	return "Click to select a bone (hold <shift> or <ctrl> to select more than one bone)"
end

function LM_SelectBone:Creator()
	return "Lost Marble"
end

function LM_SelectBone:UILabel()
	return("Select Bone")
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_SelectBone:IsEnabled(moho)
	local skel = moho:Skeleton()
	if (skel == nil) then
		if (moho.layer:LayerType() == MOHO.LT_VECTOR) then -- this tool can also be used when a vector layer is active
			skel = moho:ParentSkeleton()
		end
		if (skel == nil) then
			return false
		end
	end
	return true
end

function LM_SelectBone:OnMouseDown(moho, mouseEvent)
	self:Select(moho, mouseEvent.pt, mouseEvent.vec, mouseEvent.view, mouseEvent.shiftKey, mouseEvent.ctrlKey)
end

function LM_SelectBone:OnMouseMoved(moho, mouseEvent)
	self:Select(moho, mouseEvent.pt, mouseEvent.vec, mouseEvent.view, mouseEvent.shiftKey, mouseEvent.ctrlKey)
end

function LM_SelectBone:Select(moho, mousePt, mouseVec, mouseView, shiftSelect, ctrlSelect)
	local parentSkeleton = false
	local skel = moho:Skeleton()
	if (skel == nil) then
		if (moho.layer:LayerType() == MOHO.LT_VECTOR) then -- this tool can also be used when a vector layer is active
			skel = moho:ParentSkeleton()
			parentSkeleton = true
		end
		if (skel == nil) then
			return
		end
	end

	local id = -1
	if (parentSkeleton) then
		id = mouseView:PickBone(mousePt, mouseVec, moho.layer:Parent(), true)
	else
		id = mouseView:PickBone(mousePt, mouseVec, moho.layer, true)
	end

	if (shiftSelect) then
		if (id >= 0) then
			skel:Bone(id).fSelected = true
		end
	elseif (ctrlSelect) then
		if (id >= 0) then
			skel:Bone(id).fSelected = not skel:Bone(id).fSelected
		end
	else
		for i = 0, skel:CountBones() - 1 do
			skel:Bone(i).fSelected = (i == id)
		end
	end

	moho:UpdateBonePointSelection()
	mouseView:DrawMe()
	moho:UpdateSelectedChannels()
end

function LM_SelectBone:OnKeyDown(moho, keyEvent)
	local skel = moho:Skeleton()
	if (skel == nil) then
		if (moho.layer:LayerType() == MOHO.LT_VECTOR) then -- this tool can also be used when a vector layer is active
			skel = moho:ParentSkeleton()
		end
		if (skel == nil) then
			return
		end
	end

	if (keyEvent.keyCode == LM.GUI.KEY_UP and not keyEvent.ctrlKey) then
		for i = 0, skel:CountBones() - 1 do
			if (skel:Bone(i).fSelected) then
				local pID = skel:Bone(i).fParent
				if (pID >= 0) then
					skel:Bone(i).fSelected = false
					skel:Bone(pID).fSelected = true
				end
				break
			end
		end
		moho:UpdateBonePointSelection()
		keyEvent.view:DrawMe()
		moho:UpdateSelectedChannels()
	elseif (keyEvent.keyCode == LM.GUI.KEY_DOWN and not keyEvent.ctrlKey) then
		for i = 0, skel:CountBones() - 1 do
			if (skel:Bone(i).fSelected) then
				local childID = -1
				if (skel:CountBoneChildren(i) == 1) then
					childID = skel:GetFirstChildBone(i)
				end
				if (childID >= 0) then
					skel:Bone(i).fSelected = false
					skel:Bone(childID).fSelected = true
				end
				break
			end
		end
		moho:UpdateBonePointSelection()
		keyEvent.view:DrawMe()
		moho:UpdateSelectedChannels()
	elseif ((keyEvent.keyCode == LM.GUI.KEY_DELETE) or (keyEvent.keyCode == LM.GUI.KEY_BACKSPACE)) then
		if (moho.layer:IsBoneType()) then
			local boneLayer = moho:LayerAsGroup(moho.layer)
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			keepTrying = true
			while keepTrying do
				keepTrying = false
				for i = 0, skel:CountBones() - 1 do
					if (skel:Bone(i).fSelected) then
						for j = 0, boneLayer:CountLayers() - 1 do
							boneLayer:Layer(j):DeleteParentBone(i)
						end
						skel:DeleteBone(i)
						keepTrying = true
						break
					end
				end
			end
			keyEvent.view:DrawMe()
		end
	end
end

-- **************************************************
-- Bone constraints dialog
-- **************************************************

local LM_SelectBoneDialog = {}

function LM_SelectBoneDialog:new()
	local d = LM.GUI.SimpleDialog("Bone Constraints", LM_SelectBoneDialog)
	local l = d:GetLayout()

	l:PushV(LM.GUI.ALIGN_LEFT, 0)

	d.angleConstraints = LM.GUI.CheckBox("Angle constraints:", LM_SelectBone.DLOG_CHANGE)
	l:AddChild(d.angleConstraints, LM.GUI.ALIGN_LEFT)
	l:PushH()
		l:AddChild(LM.GUI.StaticText("Min/max (degrees)"))
		d.minAngle = LM.GUI.TextControl(0, "0000.0000", LM_SelectBone.DLOG_CHANGE, LM.GUI.FIELD_FLOAT)
		d.minAngle:SetWheelInc(5)
		l:AddChild(d.minAngle)
		d.maxAngle = LM.GUI.TextControl(0, "0000.0000", LM_SelectBone.DLOG_CHANGE, LM.GUI.FIELD_FLOAT)
		d.maxAngle:SetWheelInc(5)
		l:AddChild(d.maxAngle)
	l:Pop()

	l:AddPadding()

	l:AddChild(LM.GUI.StaticText("Angle control bone:"), LM.GUI.ALIGN_LEFT)
	l:PushH()
		d.acMenu = LM.GUI.Menu("Select Bone")
		d.acPopup = LM.GUI.PopupMenu(128, true)
		d.acPopup:SetMenu(d.acMenu)
		l:AddChild(d.acPopup)
		d.angleControl = LM.GUI.TextControl(0, "00.0000", LM_SelectBone.DLOG_CHANGE, LM.GUI.FIELD_FLOAT)
		l:AddChild(d.angleControl)
	l:Pop()

	l:AddChild(LM.GUI.StaticText("Position control bone:"), LM.GUI.ALIGN_LEFT)
	l:PushH()
		d.pcMenu = LM.GUI.Menu("Select Bone")
		d.pcPopup = LM.GUI.PopupMenu(128, true)
		d.pcPopup:SetMenu(d.pcMenu)
		l:AddChild(d.pcPopup)
		d.posControlX = LM.GUI.TextControl(0, "00.0000", LM_SelectBone.DLOG_CHANGE, LM.GUI.FIELD_FLOAT)
		l:AddChild(d.posControlX)
		d.posControlY = LM.GUI.TextControl(0, "00.0000", LM_SelectBone.DLOG_CHANGE, LM.GUI.FIELD_FLOAT)
		l:AddChild(d.posControlY)
	l:Pop()

	l:AddChild(LM.GUI.StaticText("Scale control bone:"), LM.GUI.ALIGN_LEFT)
	l:PushH()
		d.scMenu = LM.GUI.Menu("Select Bone")
		d.scPopup = LM.GUI.PopupMenu(128, true)
		d.scPopup:SetMenu(d.scMenu)
		l:AddChild(d.scPopup)
		d.scaleControl = LM.GUI.TextControl(0, "00.0000", LM_SelectBone.DLOG_CHANGE, LM.GUI.FIELD_FLOAT)
		l:AddChild(d.scaleControl)
	l:Pop()

	d.boneDynamics = LM.GUI.CheckBox("Bone dynamics:", LM_SelectBone.DLOG_CHANGE)
	l:AddChild(d.boneDynamics, LM.GUI.ALIGN_LEFT)
	l:AddChild(LM.GUI.StaticText("Torque force / Spring force / Damping force"), LM.GUI.ALIGN_LEFT)
	l:PushH()
		d.torque = LM.GUI.TextControl(0, "00.0000", LM_SelectBone.DLOG_CHANGE, LM.GUI.FIELD_UFLOAT)
		d.torque:SetWheelInc(0.1)
		l:AddChild(d.torque)
		d.spring = LM.GUI.TextControl(0, "00.0000", LM_SelectBone.DLOG_CHANGE, LM.GUI.FIELD_UFLOAT)
		d.spring:SetWheelInc(0.1)
		l:AddChild(d.spring)
		d.damping = LM.GUI.TextControl(0, "00.0000", LM_SelectBone.DLOG_CHANGE, LM.GUI.FIELD_UFLOAT)
		d.damping:SetWheelInc(0.1)
		l:AddChild(d.damping)
	l:Pop()

	l:PushH(LM.GUI.ALIGN_RIGHT, 0)
		l:AddChild(LM.GUI.Button("Close", LM.GUI.MSG_CANCEL))
	l:Pop()

	l:Pop()

	return d
end

function LM_SelectBoneDialog:UpdateWidgets()
	if (self.document and self.layer and self.skel and self.bone) then
		self.angleConstraints:Enable(true)
		self.acPopup:Enable(true)
		self.pcPopup:Enable(true)
		self.scPopup:Enable(true)
		self.boneDynamics:Enable(true)

		self.angleConstraints:SetValue(self.bone.fConstraints)
		self.minAngle:SetValue(math.deg(self.bone.fMinConstraint))
		self.maxAngle:SetValue(math.deg(self.bone.fMaxConstraint))
		self.minAngle:Enable(self.bone.fConstraints)
		self.maxAngle:Enable(self.bone.fConstraints)

		local id = self.skel:BoneID(self.bone)

		MOHO.BuildBoneChoiceMenu(self.acMenu, self.skel, LM_SelectBone.SELECTCONTROL, id)
		if (self.bone.fAngleControlParent < 0) then
			self.acMenu:SetChecked(LM_SelectBone.SELECTCONTROL, true)
		else
			self.acMenu:SetChecked(LM_SelectBone.SELECTCONTROL + 1 + self.bone.fAngleControlParent, true)
		end

		MOHO.BuildBoneChoiceMenu(self.pcMenu, self.skel, LM_SelectBone.SELECTCONTROL, id)
		if (self.bone.fPosControlParent < 0) then
			self.pcMenu:SetChecked(LM_SelectBone.SELECTCONTROL, true)
		else
			self.pcMenu:SetChecked(LM_SelectBone.SELECTCONTROL + 1 + self.bone.fPosControlParent, true)
		end

		MOHO.BuildBoneChoiceMenu(self.scMenu, self.skel, LM_SelectBone.SELECTCONTROL, id)
		if (self.bone.fScaleControlParent < 0) then
			self.scMenu:SetChecked(LM_SelectBone.SELECTCONTROL, true)
		else
			self.scMenu:SetChecked(LM_SelectBone.SELECTCONTROL + 1 + self.bone.fScaleControlParent, true)
		end

		self.angleControl:Enable(self.bone.fAngleControlParent >= 0)
		self.posControlX:Enable(self.bone.fPosControlParent >= 0)
		self.posControlY:Enable(self.bone.fPosControlParent >= 0)
		self.scaleControl:Enable(self.bone.fScaleControlParent >= 0)

		self.angleControl:SetValue(self.bone.fAngleControlScale)
		self.posControlX:SetValue(self.bone.fPosControlScale.x)
		self.posControlY:SetValue(self.bone.fPosControlScale.y)
		self.scaleControl:SetValue(self.bone.fScaleControlScale)

		self.boneDynamics:SetValue(self.bone.fBoneDynamics.value)
		self.torque:SetValue(self.bone.fTorqueForce)
		self.spring:SetValue(self.bone.fSpringForce)
		self.damping:SetValue(self.bone.fDampingForce)
		self.torque:Enable(self.bone.fBoneDynamics.value)
		self.spring:Enable(self.bone.fBoneDynamics.value)
		self.damping:Enable(self.bone.fBoneDynamics.value)
	else
		self.angleConstraints:Enable(false)
		self.minAngle:Enable(false)
		self.maxAngle:Enable(false)

		self.acPopup:Enable(false)
		self.angleControl:Enable(false)

		self.pcPopup:Enable(false)
		self.posControlX:Enable(false)
		self.posControlY:Enable(false)

		self.scPopup:Enable(false)
		self.scaleControl:Enable(false)

		self.boneDynamics:Enable(false)
		self.torque:Enable(false)
		self.spring:Enable(false)
		self.damping:Enable(false)
	end
end

function LM_SelectBoneDialog:OnOK()
	self:HandleMessage(LM_SelectBone.DLOG_CHANGE) -- send this final message in case the user is in the middle of editing some value
end

function LM_SelectBoneDialog:HandleMessage(msg)
	if (not (self.document and self.layer and self.skel and self.bone)) then
		return
	end

	if ((msg == LM_SelectBone.DLOG_CHANGE) or (msg == LM.GUI.MSG_CANCEL)) then
		self.document:PrepUndo(self.layer)
		self.document:SetDirty()

		for i = 0, self.skel:CountBones() - 1 do
			local bone = self.skel:Bone(i)
			if (bone.fSelected) then
				bone.fConstraints = self.angleConstraints:Value()
				bone.fMinConstraint = math.rad(self.minAngle:FloatValue())
				bone.fMaxConstraint = math.rad(self.maxAngle:FloatValue())
				if (self.minAngle:IsEnabled() ~= bone.fConstraints) then
					self.minAngle:Enable(bone.fConstraints)
					self.maxAngle:Enable(bone.fConstraints)
				end

				bone.fAngleControlScale = self.angleControl:FloatValue()
				bone.fPosControlScale.x = self.posControlX:FloatValue()
				bone.fPosControlScale.y = self.posControlY:FloatValue()
				bone.fScaleControlScale = self.scaleControl:FloatValue()

				if (bone.fBoneDynamics.value ~= self.boneDynamics:Value()) then
					bone.fBoneDynamics.value = self.boneDynamics:Value()
					bone.fBoneDynamics:StoreValue()
					self.layer:UpdateCurFrame(true)
					MOHO.NewKeyframe(CHANNEL_BONE_DYNAMICS)
					MOHO.NewKeyframe(CHANNEL_BONE_SEL_DYNAMICS)
				end
				bone.fTorqueForce = self.torque:FloatValue()
				bone.fSpringForce = self.spring:FloatValue()
				bone.fDampingForce = self.damping:FloatValue()
				if (self.torque:IsEnabled() ~= bone.fBoneDynamics.value) then
					self.torque:Enable(bone.fBoneDynamics.value)
					self.spring:Enable(bone.fBoneDynamics.value)
					self.damping:Enable(bone.fBoneDynamics.value)
				end
			end
		end

		MOHO.Redraw()
	elseif (msg >= LM_SelectBone.SELECTCONTROL) then
		self.document:PrepUndo(self.layer)
		self.document:SetDirty()

		for i = 0, self.skel:CountBones() - 1 do
			local bone = self.skel:Bone(i)
			if (bone.fSelected) then
				local ctrlID = self.acMenu:FirstCheckedMsg() - (LM_SelectBone.SELECTCONTROL + 1)
				if (ctrlID ~= self.skel:BoneID(bone)) then
					bone.fAngleControlParent = ctrlID
				end
				ctrlID = self.pcMenu:FirstCheckedMsg() - (LM_SelectBone.SELECTCONTROL + 1)
				if (ctrlID ~= self.skel:BoneID(bone)) then
					bone.fPosControlParent = ctrlID
				end
				ctrlID = self.scMenu:FirstCheckedMsg() - (LM_SelectBone.SELECTCONTROL + 1)
				if (ctrlID ~= self.skel:BoneID(bone)) then
					bone.fScaleControlParent = ctrlID
				end

				self.angleControl:Enable(bone.fAngleControlParent >= 0)
				self.posControlX:Enable(bone.fPosControlParent >= 0)
				self.posControlY:Enable(bone.fPosControlParent >= 0)
				self.scaleControl:Enable(bone.fScaleControlParent >= 0)
			end
		end
	end
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_SelectBone.CHANGE = MOHO.MSG_BASE
LM_SelectBone.DLOG_BEGIN = MOHO.MSG_BASE + 1
LM_SelectBone.DLOG_CHANGE = MOHO.MSG_BASE + 2
LM_SelectBone.LOCK = MOHO.MSG_BASE + 3
LM_SelectBone.DUMMY = MOHO.MSG_BASE + 4
LM_SelectBone.SELECTITEM = MOHO.MSG_BASE + 5
LM_SelectBone.SELECTCONTROL = MOHO.MSG_BASE + 300

function LM_SelectBone:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu("Select Bone")

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	self.boneName = LM.GUI.TextControl(0, "Room For a Big Long Name", self.CHANGE, LM.GUI.FIELD_TEXT)
	self.boneName:SetValue("")
	layout:AddChild(self.boneName)

	self.dlog = LM_SelectBoneDialog:new()
	self.popup = LM.GUI.PopupDialog("Bone Constraints", true, self.DLOG_BEGIN)
	self.popup:SetDialog(self.dlog)
	layout:AddChild(self.popup)

	self.lockBone = LM.GUI.CheckBox("Lock bone", self.LOCK)
	layout:AddChild(self.lockBone)
end

function LM_SelectBone:UpdateWidgets(moho)
	local skel = moho:Skeleton()
	if (skel == nil) then
		if (moho.layer:LayerType() == MOHO.LT_VECTOR) then -- this tool can also be used when a vector layer is active
			skel = moho:ParentSkeleton()
		end
		if (skel == nil) then
			return
		end
	end

	local selID = skel:SelectedBoneID()

	MOHO.BuildBoneMenu(self.menu, skel, self.SELECTITEM, self.DUMMY)

	if (selID >= 0) then
		local bone = skel:Bone(selID)
		self.boneName:SetValue(bone:Name())
		self.popup:Enable(true)
		self.dlog.bone = bone
		self.lockBone:Enable(true)
		self.lockBone:SetValue(bone.fIKLock.value)
	else
		self.boneName:SetValue("")
		self.popup:Enable(false)
		self.dlog.bone = nil
		self.lockBone:Enable(false)
		self.lockBone:SetValue(false)
	end

	self.dlog.document = moho.document
	self.dlog.layer = moho.layer
	self.dlog.skel = skel
	self.dlog:UpdateWidgets()
end

function LM_SelectBone:HandleMessage(moho, view, msg)
	local skel = moho:Skeleton()
	local layer = moho.layer
	if (skel == nil) then
		if (moho.layer:LayerType() == MOHO.LT_VECTOR) then -- this tool can also be used when a vector layer is active
			skel = moho:ParentSkeleton()
		end
		if (skel == nil) then
			return
		end
		layer = moho.layer:Parent()
	end

	if (msg == self.CHANGE) then
		if (moho:CountSelectedBones() > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, skel:CountBones() - 1 do
				local bone = skel:Bone(i)
				if (bone.fSelected) then
					bone:SetName(self.boneName:Value())
				end
			end
		end
	elseif (msg == self.LOCK) then
		if (moho:CountSelectedBones() > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, skel:CountBones() - 1 do
				local bone = skel:Bone(i)
				if (bone.fSelected) then
					if (self.lockBone:Value()) then
						skel:LockBone(i, moho.frame)
					else
						skel:UnlockBone(i, moho.frame)
					end
				end
			end
			moho.layer:UpdateCurFrame(true)
			moho:NewKeyframe(CHANNEL_BONE)
			moho:NewKeyframe(CHANNEL_BONE_LOCK)
		end
	elseif (msg == self.DLOG_BEGIN) then
		self.dlog.document = moho.document
		self.dlog.layer = layer
		self.dlog.skel = skel
		if (skel:SelectedBoneID() >= 0) then
			self.dlog.bone = skel:Bone(skel:SelectedBoneID())
		else
			self.dlog.bone = nil
		end
	elseif (msg == self.DLOG_CHANGE) then
		-- Nothing really happens here - it is a message that came from the popup dialog.
		-- However, the important thing is that this message then flows back into the Moho app, forcing a redraw.
	elseif (msg >= self.SELECTITEM and msg < self.SELECTCONTROL) then
		for i = 0, skel:CountBones() - 1 do
			skel:Bone(i).fSelected = (i == msg - self.SELECTITEM)
		end
		moho:UpdateBonePointSelection()
		moho:UpdateUI()
	end
end
