-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_SelectPoints"

-- **************************************************
-- General information about this script
-- **************************************************

LM_SelectPoints = {}

function LM_SelectPoints:Name()
	return "Select Points"
end

function LM_SelectPoints:Version()
	return "5.0"
end

function LM_SelectPoints:Description()
	return "Select/deselect points (hold <shift> to modify current selection, <ctrl> to toggle lasso mode)"
end

function LM_SelectPoints:Creator()
	return "Lost Marble"
end

function LM_SelectPoints:UILabel()
	return("Select Points")
end

function LM_SelectPoints:LoadPrefs(prefs)
	self.lassoMode = prefs:GetBool("LM_SelectPoints.lassoMode", false)
end

function LM_SelectPoints:SavePrefs(prefs)
	prefs:SetBool("LM_SelectPoints.lassoMode", self.lassoMode)
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_SelectPoints.SELMODE_GROUP_PTS = 0
LM_SelectPoints.SELMODE_PT = 1
LM_SelectPoints.SELMODE_EDGE = 2
LM_SelectPoints.SELMODE_SHAPE = 3
LM_SelectPoints.selMode = MOHO_SELMODE_GROUP_PTS
LM_SelectPoints.selRect = LM.Rect:new_local()
LM_SelectPoints.previousX = 0
LM_SelectPoints.previousY = 0
LM_SelectPoints.allowShapePicking = true

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_SelectPoints:IsEnabled(moho)
	if (moho.layer:LayerType() ~= MOHO.LT_VECTOR) then
		return false
	else
		return true
	end
end

function LM_SelectPoints:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local lassoMode = false
	if (self.lassoMode) then
		if (not(mouseEvent.ctrlKey)) then
			lassoMode = true
		end
	else
		if (mouseEvent.ctrlKey) then
			lassoMode = true
		end
	end

	self.selMode = self.SELMODE_GROUP_PTS
	if (not mouseEvent.shiftKey) then
		mesh:SelectNone()
		mouseEvent.view:DrawMe()
	end

	if (not mouseEvent.altKey) then -- enhanced selection mode
		local i = mouseEvent.view:PickPoint(mouseEvent.pt)
		if (i >= 0) then --pick a point
			self.selMode = self.SELMODE_PT
			if (mouseEvent.shiftKey) then
				mesh:Point(i).fSelected = not mesh:Point(i).fSelected
			else
				mesh:Point(i).fSelected = true
			end
		else
			local curveID = -1
			local segID = -1
			curveID, segID = mouseEvent.view:PickEdge(mouseEvent.pt, curveID, segID)
			if (curveID >= 0 and segID >= 0) then -- an edge was clicked on
				self.selMode = SELMODE_EDGE
				local curve = mesh:Curve(curveID)

				-- new method - select the points in the curve that was clicked on
				if (mouseEvent.shiftKey) then
					local isCurveSelected = true
					for i = 0, curve:CountPoints() - 1 do
						if (not curve:Point(i).fSelected) then
							isCurveSelected = false
							break
						end
					end
					for i = 0, curve:CountPoints() - 1 do
						local point = curve:Point(i)
						if ((not isCurveSelected) or (point:CountCurves() < 2)) then
							point.fSelected = not isCurveSelected
						end
					end
				else
					for i = 0, curve:CountPoints() - 1 do
						local point = curve:Point(i)
						point.fSelected = true
					end
				end

				-- old method - select all points connected to the curve that was clicked on
--[[
				-- take current selection and mark it special
				for i = 0, mesh:CountPoints() - 1 do
					local point = mesh:Point(i)
					if (point.fSelected) then
						point.fSelected = false
						point.fPrevSelected = true
					end
				end
				-- select everything connected to the picked edge
				local curve = mesh:Curve(curveID)
				curve:Point(0).fSelected = true
				mesh:SelectConnected()
				-- turn the original selection back on as well
				for i = 0, mesh:CountPoints() - 1 do
					local point = mesh:Point(i)
					if (point.fPrevSelected) then
						point.fSelected = true
						point.fPrevSelected = false
					end
				end
]]
			elseif (self.allowShapePicking) then -- try to pick a shape
				local shapeID = mouseEvent.view:PickShape(mouseEvent.pt)
				if (shapeID >= 0) then
					self.selMode = SELMODE_SHAPE
					-- take current selection and mark it special
					for i = 0, mesh:CountPoints() - 1 do
						local point = mesh:Point(i)
						if (point.fSelected) then
							point.fSelected = false
							point.fPrevSelected = true
						end
					end
					-- select everything connected to the shape
					local shape = mesh:Shape(shapeID)
					for i = 0, shape:CountEdges() - 1 do
						curveID, segID = shape:GetEdge(i, curveID, segID)
						local curve = mesh:Curve(curveID)
						curve:Point(0).fSelected = true
					end
					mesh:SelectConnected()
					-- turn the original selection back on as well
					for i = 0, mesh:CountPoints() - 1 do
						local point = mesh:Point(i)
						if (point.fPrevSelected) then
							point.fSelected = true
							point.fPrevSelected = false
						end
					end
				end
			end
		end
	end

	mouseEvent.view:DrawMe()
	if (self.selMode == self.SELMODE_GROUP_PTS) then
		if (lassoMode) then
			self.lassoList = { { mouseEvent.startPt.x, mouseEvent.startPt.y } }
			self.previousX = mouseEvent.startPt.x
			self.previousY = mouseEvent.startPt.y
		else
			self.selRect.left = mouseEvent.startPt.x
			self.selRect.top = mouseEvent.startPt.y
			self.selRect.right = mouseEvent.pt.x
			self.selRect.bottom = mouseEvent.pt.y
			mouseEvent.view:Graphics():SelectionRect(self.selRect)
		end
	end
end

function LM_SelectPoints:OnMouseMoved(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local lassoMode = false
	if (self.lassoMode) then
		if (not(mouseEvent.ctrlKey)) then
			lassoMode = true
		end
	else
		if (mouseEvent.ctrlKey) then
			lassoMode = true
		end
	end

	if (self.selMode == self.SELMODE_GROUP_PTS) then
		if (lassoMode) then
			local g = mouseEvent.view:Graphics()

			g:SetSmoothing(true)
			g:Push()
			local m = g:CurrentTransform()
			m:Invert()
			g:ApplyMatrix(m)
			g:SetColor(255, 0, 0)
			g:MoveTo(self.previousX, self.previousY)
			g:LineTo(mouseEvent.pt.x, mouseEvent.pt.y)
			g:Pop()
			g:SetSmoothing(false)
			mouseEvent.view:Refresh()

			table.insert(self.lassoList, { mouseEvent.pt.x, mouseEvent.pt.y })
			self.previousX = mouseEvent.pt.x
			self.previousY = mouseEvent.pt.y
		else
			mouseEvent.view:Graphics():SelectionRect(self.selRect)
			self.selRect.right = mouseEvent.pt.x
			self.selRect.bottom = mouseEvent.pt.y
			mouseEvent.view:Graphics():SelectionRect(self.selRect)
			mouseEvent.view:Refresh()
		end
	end
end

function LM_SelectPoints:OnMouseUp(moho, mouseEvent)
	self.allowShapePicking = true
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local lassoMode = false
	if (self.lassoMode) then
		if (not(mouseEvent.ctrlKey)) then
			lassoMode = true
		end
	else
		if (mouseEvent.ctrlKey) then
			lassoMode = true
		end
	end

	if (self.selMode == self.SELMODE_GROUP_PTS and lassoMode) then
		-- draw the finalized lasso outline
		local g = mouseEvent.view:Graphics()

		g:SetSmoothing(true)
		g:Push()
		local m = g:CurrentTransform()
		m:Invert()
		g:ApplyMatrix(m)
		g:SetColor(255, 0, 0)
		g:MoveTo(self.previousX, self.previousY)
		g:LineTo(mouseEvent.startPt.x, mouseEvent.startPt.y)
		g:Pop()
		g:SetSmoothing(false)
		mouseEvent.view:Refresh()
		LM.Snooze(100)
	end

	if (self.selMode == self.SELMODE_GROUP_PTS and mesh:CountPoints() > 0) then
		if (lassoMode) then
			local g = mouseEvent.view:Graphics()

			-- draw the lasso shape, and do hit testing
			-- 1 - draw the lasso shape
			local end1 = LM.Vector2:new_local()
			local end2 = LM.Vector2:new_local()
			g:Clear(0, 0, 0, 0)
			g:Push()
			local m = g:CurrentTransform()
			m:Invert()
			g:ApplyMatrix(m)
			g:SetColor(255, 255, 255, 255)
			g:BeginShape()
			for i = 1, table.getn(self.lassoList) - 1 do
				end1:Set(self.lassoList[i][1], self.lassoList[i][2])
				end2:Set(self.lassoList[i + 1][1], self.lassoList[i + 1][2])
				g:AddLine(end1, end2)
			end
			end1:Set(self.lassoList[table.getn(self.lassoList)][1], self.lassoList[table.getn(self.lassoList)][2])
			end2:Set(self.lassoList[1][1], self.lassoList[1][2])
			g:AddLine(end1, end2)
			g:EndShape()
			g:Pop()
-- test code to view the lasso's shape
--mouseEvent.view:Refresh()
--LM.Snooze(1000)

			-- 2 - do hit testing on the lasso shape
			local v = LM.Vector2:new_local()
			local screenPt = LM.Point:new_local()
			local m = LM.Matrix:new_local()

			moho.layer:GetFullTransform(moho.frame, m, moho.document)
			for i = 0, mesh:CountPoints() - 1 do
				local pt = mesh:Point(i)
				v:Set(pt.fPos)
				m:Transform(v)
				g:WorldToScreen(v, screenPt)
				if (g:IsFullWhite(screenPt)) then
					pt.fSelected = not pt.fSelected
				end
			end
		else
			local v = LM.Vector2:new_local()
			local screenPt = LM.Point:new_local()
			local m = LM.Matrix:new_local()

			self.selRect:Normalize()
			moho.layer:GetFullTransform(moho.frame, m, moho.document)
			for i = 0, mesh:CountPoints() - 1 do
				local pt = mesh:Point(i)
				v:Set(pt.fPos)
				m:Transform(v)
				mouseEvent.view:Graphics():WorldToScreen(v, screenPt)
				if (self.selRect:Contains(screenPt)) then
					pt.fSelected = not pt.fSelected
				end
			end
		end
	end
	mouseEvent.view:DrawMe()
	self.lassoList = nil
	moho:UpdateSelectedChannels()
end

function LM_SelectPoints:OnKeyDown(moho, keyEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if ((keyEvent.keyCode == LM.GUI.KEY_DELETE) or (keyEvent.keyCode == LM.GUI.KEY_BACKSPACE)) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()
		MOHO.DeleteSelectedPoints(mesh)
		keyEvent.view:DrawMe()
	end
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_SelectPoints.CREATE = MOHO.MSG_BASE
LM_SelectPoints.DELETE = MOHO.MSG_BASE + 1
LM_SelectPoints.LASSO = MOHO.MSG_BASE + 2
LM_SelectPoints.DUMMY = MOHO.MSG_BASE + 3
LM_SelectPoints.SELECTITEM = MOHO.MSG_BASE + 4
LM_SelectPoints.lassoMode = false

function LM_SelectPoints:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu("Select Group")

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	self.groupName = LM.GUI.TextControl(0, "Room For a Big Long Name", 0, LM.GUI.FIELD_TEXT)
	self.groupName:SetValue("")
	layout:AddChild(self.groupName)

	layout:AddChild(LM.GUI.Button("Create", self.CREATE))
	layout:AddChild(LM.GUI.Button("Delete", self.DELETE))

	self.lassoCheck = LM.GUI.CheckBox("Lasso mode", self.LASSO)
	layout:AddChild(self.lassoCheck)
end

function LM_SelectPoints:UpdateWidgets(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	MOHO.BuildGroupMenu(self.menu, mesh, self.SELECTITEM, self.DUMMY)

	self.lassoCheck:SetValue(self.lassoMode)
end

function LM_SelectPoints:HandleMessage(moho, view, msg)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (msg == self.CREATE) then
		local name = self.groupName:Value()

		if (string.len(name) == 0) then
			LM.GUI.Alert(LM.GUI.ALERT_INFO, "To create a group, you must first name it.")
		elseif (moho:CountSelectedPoints() < 1) then
			LM.GUI.Alert(LM.GUI.ALERT_INFO, "Can't create group - no points are selected.")
		else
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			mesh:AddGroup(name)
			moho:UpdateUI()
		end
	elseif (msg == self.DELETE) then
		local name = self.groupName:Value()
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()
		mesh:DeleteGroup(name)
		self.groupName:SetValue("")
		moho:UpdateUI()
	elseif (msg == self.LASSO) then
		self.lassoMode = self.lassoCheck:Value()
	elseif (msg >= self.SELECTITEM) then
		mesh:SelectNone()
		local i = msg - self.SELECTITEM
		local name = mesh:Group(i):Name()
		mesh:SelectGroup(name)
		self.groupName:SetValue(name)
		moho:UpdateUI()
	end
end
